@@ -173,8 +173,11 @@ protected:
         QStringList lines = str.split(QLatin1Char('\n'));
         bool multiline = lines.length() > 1;
         for (int i = 0; i < lines.size(); ++i) {
-            if (multiline)
+            if (multiline) {
+                if (i == 0)
+                    newLine();
                 _line = lines.at(i);  // multiline comments don't keep track of previos lines
+            }
             else
                 _line += lines.at(i);
             if (i != lines.size() - 1)
@@ -187,7 +190,7 @@ protected:
     {
         SourceLocation fixedLoc = commentLoc;
         fixCommentLocation(fixedLoc);
-        if (precededByEmptyLine(fixedLoc))
+        if (precededByEmptyLine(fixedLoc) && !_result.endsWith(QLatin1String("\n\n")))
             newLine();
         outCommentText(toString(fixedLoc)); // don't use the sourceloc overload here
         if (followedByNewLine(fixedLoc))
@@ -528,7 +531,6 @@ protected:
     bool visit(UiPragma *ast) override
     {
         out("pragma ", ast->pragmaToken);
-        accept(ast->pragmaType);
         return false;
     }
 
@@ -663,20 +665,16 @@ protected:
     bool visit(NumericLiteral *ast) override { out(ast->literalToken); return true; }
     bool visit(RegExpLiteral *ast) override { out(ast->literalToken); return true; }
 
-    bool visit(ArrayLiteral *ast) override
+    bool visit(ArrayPattern *ast) override
     {
         out(ast->lbracketToken);
         if (ast->elements)
             accept(ast->elements);
-        if (ast->elements && ast->elision)
-            out(", ", ast->commaToken);
-        if (ast->elision)
-            accept(ast->elision);
         out(ast->rbracketToken);
         return false;
     }
 
-    bool visit(ObjectLiteral *ast) override
+    bool visit(ObjectPattern *ast) override
     {
         out(ast->lbraceToken);
         lnAcceptIndented(ast->properties);
@@ -685,55 +683,57 @@ protected:
         return false;
     }
 
-    bool visit(ElementList *ast) override
+    bool visit(PatternElementList *ast) override
     {
-        for (ElementList *it = ast; it; it = it->next) {
+        for (PatternElementList *it = ast; it; it = it->next) {
             if (it->elision)
                 accept(it->elision);
-            if (it->elision && it->expression)
+            if (it->elision && it->element)
                 out(", ");
-            if (it->expression)
-                accept(it->expression);
+            if (it->element)
+                accept(it->element);
             if (it->next)
-                out(", ", ast->commaToken);
+                out(", ");
         }
         return false;
     }
 
-    bool visit(PropertyAssignmentList *ast) override
+    bool visit(PatternPropertyList *ast) override
     {
-        for (PropertyAssignmentList *it = ast; it; it = it->next) {
-            PropertyNameAndValue *assignment = AST::cast<PropertyNameAndValue *>(it->assignment);
+        for (PatternPropertyList *it = ast; it; it = it->next) {
+            PatternProperty *assignment = AST::cast<PatternProperty *>(it->property);
             if (assignment) {
                 out("\"");
                 accept(assignment->name);
                 out("\"");
                 out(": ", assignment->colonToken);
-                accept(assignment->value);
+                accept(assignment->initializer);
                 if (it->next) {
-                    out(",", ast->commaToken); // always invalid?
+                    out(","); // always invalid?
                     newLine();
                 }
                 continue;
             }
-            PropertyGetterSetter *getterSetter = AST::cast<PropertyGetterSetter *>(it->assignment);
-            if (getterSetter) {
-                switch (getterSetter->type) {
-                case PropertyGetterSetter::Getter:
+            PatternPropertyList *getterSetter = AST::cast<PatternPropertyList *>(it->next);
+            if (getterSetter->property) {
+                switch (getterSetter->property->type) {
+                case PatternElement::Getter:
                     out("get");
                     break;
-                case PropertyGetterSetter::Setter:
+                case PatternElement::Setter:
                     out("set");
                     break;
+                default:
+                    break;
                 }
 
-                accept(getterSetter->name);
-                out("(", getterSetter->lparenToken);
-                accept(getterSetter->formals);
-                out("(", getterSetter->rparenToken);
-                out(" {", getterSetter->lbraceToken);
-                accept(getterSetter->functionBody);
-                out(" }", getterSetter->rbraceToken);
+                accept(getterSetter->property->name);
+                out("(");
+                //accept(getterSetter->formals);  // TODO
+                out(")");
+                out(" {");
+                //accept(getterSetter->functionBody);  // TODO
+                out(" }");
             }
         }
         return false;
@@ -919,12 +919,14 @@ protected:
         return false;
     }
 
-    bool visit(VariableDeclaration *ast) override
+    bool visit(PatternElement *ast) override
     {
+
         out(ast->identifierToken);
-        if (ast->expression) {
-            out(" = ");
-            accept(ast->expression);
+        if (ast->initializer) {
+            if (ast->isVariableDeclaration())
+                out(" = ");
+            accept(ast->initializer);
         }
         return false;
     }
@@ -993,45 +995,13 @@ protected:
         return false;
     }
 
-    bool visit(LocalForStatement *ast) override
-    {
-        out(ast->forToken);
-        out(" ");
-        out(ast->lparenToken);
-        out(ast->varToken);
-        out(" ");
-        accept(ast->declarations);
-        out("; ", ast->firstSemicolonToken);
-        accept(ast->condition);
-        out("; ", ast->secondSemicolonToken);
-        accept(ast->expression);
-        out(")", ast->rparenToken);
-        acceptBlockOrIndented(ast->statement);
-        return false;
-    }
-
     bool visit(ForEachStatement *ast) override
     {
         out(ast->forToken);
         out(" ");
         out(ast->lparenToken);
-        accept(ast->initialiser);
-        out(" in ", ast->inToken);
-        accept(ast->expression);
-        out(ast->rparenToken);
-        acceptBlockOrIndented(ast->statement);
-        return false;
-    }
-
-    bool visit(LocalForEachStatement *ast) override
-    {
-        out(ast->forToken);
-        out(" ");
-        out(ast->lparenToken);
-        out(ast->varToken);
-        out(" ");
-        accept(ast->declaration);
-        out(" in ", ast->inToken);
+        accept(ast->lhs);
+        out(" in ");
         accept(ast->expression);
         out(ast->rparenToken);
         acceptBlockOrIndented(ast->statement);
@@ -1242,12 +1212,6 @@ protected:
         return false;
     }
 
-    bool visit(UiQualifiedPragmaId *ast) override
-    {
-        out(ast->identifierToken);
-        return false;
-    }
-
     bool visit(Elision *ast) override
     {
         for (Elision *it = ast; it; it = it->next) {
@@ -1285,16 +1249,6 @@ protected:
         return false;
     }
 
-    bool visit(SourceElements *ast) override
-    {
-        for (SourceElements *it = ast; it; it = it->next) {
-            accept(it->element);
-            if (it->next)
-                newLine();
-        }
-        return false;
-    }
-
     bool visit(VariableDeclarationList *ast) override
     {
         for (VariableDeclarationList *it = ast; it; it = it->next) {
@@ -1318,9 +1272,7 @@ protected:
     bool visit(FormalParameterList *ast) override
     {
         for (FormalParameterList *it = ast; it; it = it->next) {
-            if (it->commaToken.isValid())
-                out(", ", it->commaToken);
-            out(it->identifierToken);
+            out(it->element->bindingIdentifier.toString()); // TODO
         }
         return false;
     }
