@@ -23,15 +23,16 @@
 **
 ****************************************************************************/
 
-#ifndef SQLSTATEMENTBUILDEREXCEPTION_H
-#define SQLSTATEMENTBUILDEREXCEPTION_H
+#pragma once
 
 #include "sqliteexception.h"
 
-class SQLITE_EXPORT SqlStatementBuilderException : public SqliteException
+namespace Sqlite {
+
+class SQLITE_EXPORT SqlStatementBuilderException : public Exception
 {
 public:
-    SqlStatementBuilderException(const char *whatErrorHasHappen, const char *errorMessage = 0);
+    using Exception::Exception;
 };
 
-#endif // SQLSTATEMENTBUILDEREXCEPTION_H
+} // namespace Sqlite
