@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef SSHREMOTEPROCESSRUNNER_H
-#define SSHREMOTEPROCESSRUNNER_H
+#pragma once
 
 #include "sshconnection.h"
 #include "sshremoteprocess.h"
@@ -41,19 +40,15 @@ public:
     ~SshRemoteProcessRunner();
 
     void run(const QByteArray &command, const SshConnectionParameters &sshParams);
-    void runInTerminal(const QByteArray &command, const SshPseudoTerminal &terminal,
-        const SshConnectionParameters &sshParams);
+    void runInTerminal(const QByteArray &command, const SshConnectionParameters &sshParams);
     QByteArray command() const;
 
-    QSsh::SshError lastConnectionError() const;
     QString lastConnectionErrorString() const;
 
     bool isProcessRunning() const;
     void writeDataToProcess(const QByteArray &data);
-    void sendSignalToProcess(SshRemoteProcess::Signal signal); // No effect with OpenSSH server.
-    void cancel(); // Does not stop remote process, just frees SSH-related process resources.
+    void cancel();
     SshRemoteProcess::ExitStatus processExitStatus() const;
-    SshRemoteProcess::Signal processExitSignal() const;
     int processExitCode() const;
     QString processErrorString() const;
     QByteArray readAllStandardOutput();
@@ -64,18 +59,16 @@ signals:
     void processStarted();
     void readyReadStandardOutput();
     void readyReadStandardError();
-    void processClosed(int exitStatus); // values are of type SshRemoteProcess::ExitStatus
-
-private slots:
-    void handleConnected();
-    void handleConnectionError(QSsh::SshError error);
-    void handleDisconnected();
-    void handleProcessStarted();
-    void handleProcessFinished(int exitStatus);
-    void handleStdout();
-    void handleStderr();
+    void processClosed(const QString &error);
 
 private:
+    void handleConnected();
+    void handleConnectionError();
+    void handleDisconnected();
+    void handleProcessStarted();
+    void handleProcessFinished(const QString &error);
+    void handleStdout();
+    void handleStderr();
     void runInternal(const QByteArray &command, const QSsh::SshConnectionParameters &sshParams);
     void setState(int newState);
 
@@ -83,5 +76,3 @@ private:
 };
 
 } // namespace QSsh
-
-#endif // SSHREMOTEPROCESSRUNNER_H
