@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,38 +9,34 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "androidmanifesteditorwidget.h"
 #include "androidmanifesteditor.h"
+#include "androidconfigurations.h"
 #include "androidconstants.h"
 #include "androidmanifestdocument.h"
 #include "androidmanager.h"
-#include "androidqtsupport.h"
 
-#include <coreplugin/coreconstants.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/infobar.h>
 #include <coreplugin/editormanager/ieditor.h>
 
+#include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/project.h>
+#include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projectwindow.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
@@ -50,6 +46,7 @@
 #include <texteditor/texteditoractionhandler.h>
 #include <texteditor/texteditor.h>
 #include <utils/algorithm.h>
+#include <utils/utilsicons.h>
 
 #include <QLineEdit>
 #include <QFileInfo>
@@ -70,7 +67,9 @@
 #include <QFileDialog>
 #include <QTimer>
 #include <QCheckBox>
+#include <QScrollArea>
 
+#include <algorithm>
 #include <limits>
 
 using namespace ProjectExplorer;
@@ -80,17 +79,15 @@ using namespace Android::Internal;
 namespace {
 const QLatin1String packageNameRegExp("^([a-z]{1}[a-z0-9_]+(\\.[a-zA-Z]{1}[a-zA-Z0-9_]*)*)$");
 const char infoBarId[] = "Android.AndroidManifestEditor.InfoBar";
-const char androidManifestEditorGeneralPaneContextId[] = "AndroidManifestEditorWidget.GeneralWidget";
 
 bool checkPackageName(const QString &packageName)
 {
     return QRegExp(packageNameRegExp).exactMatch(packageName);
 }
 
-Project *androidProject(const QString &file)
+Project *androidProject(const Utils::FileName &fileName)
 {
-    Utils::FileName fileName = Utils::FileName::fromString(file);
-    foreach (Project *project, SessionManager::projects()) {
+    for (Project *project : SessionManager::projects()) {
         if (!project->activeTarget())
             continue;
         Kit *kit = project->activeTarget()->kit();
@@ -98,20 +95,17 @@ Project *androidProject(const QString &file)
                 && fileName.isChildOf(project->projectDirectory()))
             return project;
     }
-    return 0;
+    return nullptr;
 }
 
 } // anonymous namespace
 
 AndroidManifestEditorWidget::AndroidManifestEditorWidget()
-    : QScrollArea(),
+    : QStackedWidget(),
       m_dirty(false),
-      m_stayClean(false),
-      m_setAppName(false),
-      m_appNameInStringsXml(false)
+      m_stayClean(false)
 {
     m_textEditorWidget = new AndroidManifestTextEditorWidget(this);
-    m_textEditorWidget->setupAsPlainEditor();
 
     initializePage();
 
@@ -120,35 +114,30 @@ AndroidManifestEditorWidget::AndroidManifestEditorWidget()
 
     m_editor = new AndroidManifestEditor(this);
 
-    setWidgetResizable(true);
+    connect(&m_timerParseCheck, &QTimer::timeout,
+            this, &AndroidManifestEditorWidget::delayedParseCheck);
 
-    connect(&m_timerParseCheck, SIGNAL(timeout()),
-            this, SLOT(delayedParseCheck()));
-
-    connect(m_textEditorWidget->document(), SIGNAL(contentsChanged()),
-            this, SLOT(startParseCheck()));
+    connect(m_textEditorWidget->document(), &QTextDocument::contentsChanged,
+            this, &AndroidManifestEditorWidget::startParseCheck);
+    connect(m_textEditorWidget->textDocument(), &TextEditor::TextDocument::reloadFinished,
+            this, [this](bool success) { if (success) updateAfterFileLoad(); });
+    connect(m_textEditorWidget->textDocument(), &TextEditor::TextDocument::openFinishedSuccessfully,
+            this, &AndroidManifestEditorWidget::updateAfterFileLoad);
 }
 
 void AndroidManifestEditorWidget::initializePage()
 {
-    m_stackedWidget = new QStackedWidget(this);
-    setWidget(m_stackedWidget);
-
-    Core::IContext *myContext = new Core::IContext(this);
-    myContext->setWidget(m_stackedWidget);
-    myContext->setContext(Core::Context(androidManifestEditorGeneralPaneContextId)); // where is the context used?
-    Core::ICore::addContextObject(myContext);
-
     QWidget *mainWidget = new QWidget; // different name
 
-    QVBoxLayout *topLayout = new QVBoxLayout(mainWidget);
+    auto topLayout = new QVBoxLayout(mainWidget);
 
-    QGroupBox *packageGroupBox = new QGroupBox(mainWidget);
+    auto packageGroupBox = new QGroupBox(mainWidget);
     topLayout->addWidget(packageGroupBox);
 
+    auto setDirtyFunc = [this] { setDirty(); };
     packageGroupBox->setTitle(tr("Package"));
     {
-        QFormLayout *formLayout = new QFormLayout();
+        auto formLayout = new QFormLayout();
 
         m_packageNameLineEdit = new QLineEdit(packageGroupBox);
         m_packageNameLineEdit->setToolTip(tr(
@@ -170,23 +159,19 @@ void AndroidManifestEditorWidget::initializePage()
         m_packageNameWarning->setVisible(false);
 
         m_packageNameWarningIcon = new QLabel;
-        m_packageNameWarningIcon->setPixmap(QPixmap(QLatin1String(Core::Constants::ICON_WARNING)));
+        m_packageNameWarningIcon->setPixmap(Utils::Icons::WARNING.pixmap());
         m_packageNameWarningIcon->setVisible(false);
         m_packageNameWarningIcon->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 
-        QHBoxLayout *warningRow = new QHBoxLayout;
+        auto warningRow = new QHBoxLayout;
         warningRow->setMargin(0);
         warningRow->addWidget(m_packageNameWarningIcon);
         warningRow->addWidget(m_packageNameWarning);
 
         formLayout->addRow(QString(), warningRow);
 
-
-        m_versionCode = new QSpinBox(packageGroupBox);
-        m_versionCode->setMaximum(std::numeric_limits<int>::max());
-        m_versionCode->setValue(1);
-        m_versionCode->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
-        formLayout->addRow(tr("Version code:"), m_versionCode);
+        m_versionCodeLineEdit = new QLineEdit(packageGroupBox);
+        formLayout->addRow(tr("Version code:"), m_versionCodeLineEdit);
 
         m_versionNameLinedit = new QLineEdit(packageGroupBox);
         formLayout->addRow(tr("Version name:"), m_versionNameLinedit);
@@ -208,37 +193,44 @@ void AndroidManifestEditorWidget::initializePage()
 
         packageGroupBox->setLayout(formLayout);
 
-        connect(m_packageNameLineEdit, SIGNAL(textEdited(QString)),
-                this, SLOT(setPackageName()));
-        connect(m_versionCode, SIGNAL(valueChanged(int)),
-                this, SLOT(setDirty()));
-        connect(m_versionNameLinedit, SIGNAL(textEdited(QString)),
-                this, SLOT(setDirty()));
-        connect(m_androidMinSdkVersion, SIGNAL(currentIndexChanged(int)),
-                this, SLOT(setDirty()));
-        connect(m_androidTargetSdkVersion, SIGNAL(currentIndexChanged(int)),
-                this, SLOT(setDirty()));
+        updateSdkVersions();
+
+        connect(m_packageNameLineEdit, &QLineEdit::textEdited,
+                this, &AndroidManifestEditorWidget::setPackageName);
+        connect(m_versionCodeLineEdit, &QLineEdit::textEdited,
+                 this, setDirtyFunc);
+        connect(m_versionNameLinedit, &QLineEdit::textEdited,
+                this, setDirtyFunc);
+        connect(m_androidMinSdkVersion,
+                static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+                this, setDirtyFunc);
+        connect(m_androidTargetSdkVersion,
+                static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+                this, setDirtyFunc);
 
     }
 
     // Application
-    QGroupBox *applicationGroupBox = new QGroupBox(mainWidget);
+    auto applicationGroupBox = new QGroupBox(mainWidget);
     topLayout->addWidget(applicationGroupBox);
 
     applicationGroupBox->setTitle(tr("Application"));
     {
-        QFormLayout *formLayout = new QFormLayout();
+        auto formLayout = new QFormLayout();
 
         m_appNameLineEdit = new QLineEdit(applicationGroupBox);
         formLayout->addRow(tr("Application name:"), m_appNameLineEdit);
 
+        m_activityNameLineEdit = new QLineEdit(applicationGroupBox);
+        formLayout->addRow(tr("Activity name:"), m_activityNameLineEdit);
+
         m_targetLineEdit = new QComboBox(applicationGroupBox);
         m_targetLineEdit->setEditable(true);
         m_targetLineEdit->setDuplicatesEnabled(true);
         m_targetLineEdit->installEventFilter(this);
         formLayout->addRow(tr("Run:"), m_targetLineEdit);
 
-        QHBoxLayout *iconLayout = new QHBoxLayout();
+        auto iconLayout = new QHBoxLayout();
         m_lIconButton = new QToolButton(applicationGroupBox);
         m_lIconButton->setMinimumSize(QSize(48, 48));
         m_lIconButton->setMaximumSize(QSize(48, 48));
@@ -265,24 +257,29 @@ void AndroidManifestEditorWidget::initializePage()
 
         applicationGroupBox->setLayout(formLayout);
 
-        connect(m_appNameLineEdit, SIGNAL(textEdited(QString)),
-                this, SLOT(setAppName()));
-        connect(m_targetLineEdit, SIGNAL(currentTextChanged(QString)),
-                this, SLOT(setDirty()));
+        connect(m_appNameLineEdit, &QLineEdit::textEdited,
+                this, setDirtyFunc);
+        connect(m_activityNameLineEdit, &QLineEdit::textEdited,
+                this, setDirtyFunc);
+        connect(m_targetLineEdit, &QComboBox::currentTextChanged,
+                this, setDirtyFunc);
 
-        connect(m_lIconButton, SIGNAL(clicked()), SLOT(setLDPIIcon()));
-        connect(m_mIconButton, SIGNAL(clicked()), SLOT(setMDPIIcon()));
-        connect(m_hIconButton, SIGNAL(clicked()), SLOT(setHDPIIcon()));
+        connect(m_lIconButton, &QAbstractButton::clicked,
+                this, &AndroidManifestEditorWidget::setLDPIIcon);
+        connect(m_mIconButton, &QAbstractButton::clicked,
+                this, &AndroidManifestEditorWidget::setMDPIIcon);
+        connect(m_hIconButton, &QAbstractButton::clicked,
+                this, &AndroidManifestEditorWidget::setHDPIIcon);
     }
 
 
     // Permissions
-    QGroupBox *permissionsGroupBox = new QGroupBox(mainWidget);
+    auto permissionsGroupBox = new QGroupBox(mainWidget);
     topLayout->addWidget(permissionsGroupBox);
 
     permissionsGroupBox->setTitle(tr("Permissions"));
     {
-        QGridLayout *layout = new QGridLayout(permissionsGroupBox);
+        auto layout = new QGridLayout(permissionsGroupBox);
 
         m_defaultPermissonsCheckBox = new QCheckBox(this);
         m_defaultPermissonsCheckBox->setText(tr("Include default permissions for Qt modules."));
@@ -445,44 +442,63 @@ void AndroidManifestEditorWidget::initializePage()
 
         permissionsGroupBox->setLayout(layout);
 
-        connect(m_defaultPermissonsCheckBox, SIGNAL(stateChanged(int)),
-                this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
-        connect(m_defaultFeaturesCheckBox, SIGNAL(stateChanged(int)),
-                this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
+        connect(m_defaultPermissonsCheckBox, &QCheckBox::stateChanged,
+                this, &AndroidManifestEditorWidget::defaultPermissionOrFeatureCheckBoxClicked);
+        connect(m_defaultFeaturesCheckBox, &QCheckBox::stateChanged,
+                this, &AndroidManifestEditorWidget::defaultPermissionOrFeatureCheckBoxClicked);
 
-        connect(m_addPermissionButton, SIGNAL(clicked()),
-                this, SLOT(addPermission()));
-        connect(m_removePermissionButton, SIGNAL(clicked()),
-                this, SLOT(removePermission()));
-        connect(m_permissionsComboBox, SIGNAL(currentTextChanged(QString)),
-                this, SLOT(updateAddRemovePermissionButtons()));
+        connect(m_addPermissionButton, &QAbstractButton::clicked,
+                this, &AndroidManifestEditorWidget::addPermission);
+        connect(m_removePermissionButton, &QAbstractButton::clicked,
+                this, &AndroidManifestEditorWidget::removePermission);
+        connect(m_permissionsComboBox, &QComboBox::currentTextChanged,
+                this, &AndroidManifestEditorWidget::updateAddRemovePermissionButtons);
     }
 
     topLayout->addSpacerItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::MinimumExpanding));
 
-    m_stackedWidget->insertWidget(General, mainWidget);
-    m_stackedWidget->insertWidget(Source, m_textEditorWidget);
+    auto mainWidgetScrollArea = new QScrollArea;
+    mainWidgetScrollArea->setWidgetResizable(true);
+    mainWidgetScrollArea->setWidget(mainWidget);
+    mainWidgetScrollArea->setFocusProxy(m_packageNameLineEdit);
+
+    insertWidget(General, mainWidgetScrollArea);
+    insertWidget(Source, m_textEditorWidget);
 }
 
 bool AndroidManifestEditorWidget::eventFilter(QObject *obj, QEvent *event)
 {
     if (obj == m_targetLineEdit) {
         if (event->type() == QEvent::FocusIn)
-            QTimer::singleShot(0, this, SLOT(updateTargetComboBox()));
+            QTimer::singleShot(0, this, &AndroidManifestEditorWidget::updateTargetComboBox);
     }
 
     return QWidget::eventFilter(obj, event);
 }
 
+void AndroidManifestEditorWidget::focusInEvent(QFocusEvent *event)
+{
+    if (currentWidget()) {
+        if (currentWidget()->focusWidget())
+            currentWidget()->focusWidget()->setFocus(event->reason());
+        else
+            currentWidget()->setFocus(event->reason());
+    }
+}
+
 void AndroidManifestEditorWidget::updateTargetComboBox()
 {
-    const QString docPath(m_textEditorWidget->textDocument()->filePath());
-    ProjectExplorer::Project *project = androidProject(docPath);
+    Project *project = androidProject(m_textEditorWidget->textDocument()->filePath());
     QStringList items;
     if (project) {
-        ProjectExplorer::Kit *kit = project->activeTarget()->kit();
-        if (ProjectExplorer::DeviceTypeKitInformation::deviceTypeId(kit) == Constants::ANDROID_DEVICE_TYPE)
-            items = AndroidManager::androidQtSupport(project->activeTarget())->projectTargetApplications(project->activeTarget());
+        Kit *kit = project->activeTarget()->kit();
+        if (DeviceTypeKitInformation::deviceTypeId(kit) == Constants::ANDROID_DEVICE_TYPE) {
+            ProjectNode *root = project->rootProjectNode();
+            root->forEachProjectNode([&items](const ProjectNode *projectNode) {
+                items << projectNode->targetApplications();
+            });
+            items.sort();
+        }
     }
 
     // QComboBox randomly resets what the user has entered
@@ -498,15 +514,8 @@ void AndroidManifestEditorWidget::updateTargetComboBox()
     m_targetLineEdit->addItems(items);
 }
 
-bool AndroidManifestEditorWidget::open(QString *errorString, const QString &fileName, const QString &realFileName)
+void AndroidManifestEditorWidget::updateAfterFileLoad()
 {
-    bool result = m_textEditorWidget->open(errorString, fileName, realFileName);
-
-    updateSdkVersions();
-
-    if (!result)
-        return result;
-
     QString error;
     int errorLine;
     int errorColumn;
@@ -515,13 +524,12 @@ bool AndroidManifestEditorWidget::open(QString *errorString, const QString &file
         if (checkDocument(doc, &error, &errorLine, &errorColumn)) {
             if (activePage() != Source)
                 syncToWidgets(doc);
-            return true;
+            return;
         }
     }
     // some error occurred
     updateInfoBar(error, errorLine, errorColumn);
     setActivePage(Source);
-    return true;
 }
 
 void AndroidManifestEditorWidget::setDirty(bool dirty)
@@ -537,13 +545,12 @@ bool AndroidManifestEditorWidget::isModified() const
     return m_dirty
             || !m_hIconPath.isEmpty()
             || !m_mIconPath.isEmpty()
-            || !m_lIconPath.isEmpty()
-            || m_setAppName;
+            || !m_lIconPath.isEmpty();
 }
 
 AndroidManifestEditorWidget::EditorPage AndroidManifestEditorWidget::activePage() const
 {
-    return AndroidManifestEditorWidget::EditorPage(m_stackedWidget->currentIndex());
+    return AndroidManifestEditorWidget::EditorPage(currentIndex());
 }
 
 bool AndroidManifestEditorWidget::setActivePage(EditorPage page)
@@ -555,19 +562,20 @@ bool AndroidManifestEditorWidget::setActivePage(EditorPage page)
 
     if (page == Source) {
         syncToEditor();
-        setFocus();
     } else {
         if (!syncToWidgets())
             return false;
-// TODO?
-//        QWidget *fw = m_overlayWidget->focusWidget();
-//        if (fw && fw != m_overlayWidget)
-//            fw->setFocus();
-//        else
-//            m_packageNameLineEdit->setFocus();
     }
 
-    m_stackedWidget->setCurrentIndex(page);
+    setCurrentIndex(page);
+
+    QWidget *cw = currentWidget();
+    if (cw) {
+        if (cw->focusWidget())
+            cw->focusWidget()->setFocus();
+        else
+            cw->setFocus();
+    }
     return true;
 }
 
@@ -576,32 +584,7 @@ void AndroidManifestEditorWidget::preSave()
     if (activePage() != Source)
         syncToEditor();
 
-    if (m_setAppName && m_appNameInStringsXml) {
-        QString baseDir = QFileInfo(m_textEditorWidget->textDocument()->filePath()).absolutePath();
-        QString fileName = baseDir + QLatin1String("/res/values/strings.xml");
-        QFile f(fileName);
-        if (f.open(QIODevice::ReadOnly)) {
-            QDomDocument doc;
-            if (doc.setContent(f.readAll())) {
-                QDomElement metadataElem = doc.documentElement().firstChildElement(QLatin1String("string"));
-                while (!metadataElem.isNull()) {
-                    if (metadataElem.attribute(QLatin1String("name")) == QLatin1String("app_name")) {
-                        metadataElem.removeChild(metadataElem.firstChild());
-                        metadataElem.appendChild(doc.createTextNode(m_appNameLineEdit->text()));
-                        break;
-                    }
-                    metadataElem = metadataElem.nextSiblingElement(QLatin1String("string"));
-                }
-
-                f.close();
-                f.open(QIODevice::WriteOnly);
-                f.write(doc.toByteArray((4)));
-            }
-        }
-        m_setAppName = false;
-    }
-
-    QString baseDir = QFileInfo(m_textEditorWidget->textDocument()->filePath()).absolutePath();
+    QString baseDir = m_textEditorWidget->textDocument()->filePath().toFileInfo().absolutePath();
     if (!m_lIconPath.isEmpty()) {
         copyIcon(LowDPI, baseDir, m_lIconPath);
         m_lIconPath.clear();
@@ -619,6 +602,25 @@ void AndroidManifestEditorWidget::preSave()
     updateInfoBar();
 }
 
+void AndroidManifestEditorWidget::postSave()
+{
+    const Utils::FileName docPath = m_textEditorWidget->textDocument()->filePath();
+    ProjectExplorer::Project *project = androidProject(docPath);
+    if (project) {
+        if (Target *target = project->activeTarget()) {
+            if (BuildConfiguration *bc = target->activeBuildConfiguration()) {
+                QString androidNdkPlatform = AndroidConfigurations::currentConfig()
+                        .bestNdkPlatformMatch(AndroidManager::minimumSDK(target));
+                if (m_androidNdkPlatform != androidNdkPlatform) {
+                    m_androidNdkPlatform = androidNdkPlatform;
+                    bc->updateCacheAndEmitEnvironmentChanged();
+                    bc->regenerateBuildFiles(nullptr);
+                }
+            }
+        }
+    }
+}
+
 Core::IEditor *AndroidManifestEditorWidget::editor() const
 {
     return m_editor;
@@ -646,7 +648,8 @@ bool AndroidManifestEditorWidget::syncToWidgets()
     return false;
 }
 
-bool AndroidManifestEditorWidget::checkDocument(QDomDocument doc, QString *errorMessage, int *errorLine, int *errorColumn)
+bool AndroidManifestEditorWidget::checkDocument(const QDomDocument &doc, QString *errorMessage,
+                                                int *errorLine, int *errorColumn)
 {
     QDomElement manifest = doc.documentElement();
     if (manifest.tagName() != QLatin1String("manifest")) {
@@ -695,7 +698,6 @@ void AndroidManifestEditorWidget::updateInfoBar()
 
 void AndroidManifestEditorWidget::updateSdkVersions()
 {
-    const QString docPath(m_textEditorWidget->textDocument()->filePath());
     QPair<int, int> apiLevels = AndroidManager::apiLevelRange();
     for (int i = apiLevels.first; i < apiLevels.second + 1; ++i)
         m_androidMinSdkVersion->addItem(tr("API %1: %2")
@@ -759,38 +761,23 @@ void AndroidManifestEditorWidget::syncToWidgets(const QDomDocument &doc)
     m_stayClean = true;
     QDomElement manifest = doc.documentElement();
     m_packageNameLineEdit->setText(manifest.attribute(QLatin1String("package")));
-    m_versionCode->setValue(manifest.attribute(QLatin1String("android:versionCode")).toInt());
+    m_versionCodeLineEdit->setText(manifest.attribute(QLatin1String("android:versionCode")));
     m_versionNameLinedit->setText(manifest.attribute(QLatin1String("android:versionName")));
 
     QDomElement usesSdkElement = manifest.firstChildElement(QLatin1String("uses-sdk"));
     setApiLevel(m_androidMinSdkVersion, usesSdkElement, QLatin1String("android:minSdkVersion"));
     setApiLevel(m_androidTargetSdkVersion, usesSdkElement, QLatin1String("android:targetSdkVersion"));
 
-    QString baseDir = QFileInfo(m_textEditorWidget->textDocument()->filePath()).absolutePath();
-    QString fileName = baseDir + QLatin1String("/res/values/strings.xml");
+    QString baseDir = m_textEditorWidget->textDocument()->filePath().toFileInfo().absolutePath();
 
     QDomElement applicationElement = manifest.firstChildElement(QLatin1String("application"));
+    m_appNameLineEdit->setText(applicationElement.attribute(QLatin1String("android:label")));
 
-    QFile f(fileName);
-    if (f.exists() && f.open(QIODevice::ReadOnly)) {
-        QDomDocument doc;
-        if (doc.setContent(&f)) {
-            QDomElement metadataElem = doc.documentElement().firstChildElement(QLatin1String("string"));
-            while (!metadataElem.isNull()) {
-                if (metadataElem.attribute(QLatin1String("name")) == QLatin1String("app_name")) {
-                    m_appNameLineEdit->setText(metadataElem.text());
-                    break;
-                }
-                metadataElem = metadataElem.nextSiblingElement(QLatin1String("string"));
-            }
-        }
-        m_appNameInStringsXml = true;
-    } else {
-        m_appNameLineEdit->setText(applicationElement.attribute(QLatin1String("android:label")));
-        m_appNameInStringsXml = false;
-    }
+    QDomElement activityElem = applicationElement.firstChildElement(QLatin1String("activity"));
+    m_activityNameLineEdit->setText(activityElem.attribute(QLatin1String("android:label")));
+
+    QDomElement metadataElem = activityElem.firstChildElement(QLatin1String("meta-data"));
 
-    QDomElement metadataElem = applicationElement.firstChildElement(QLatin1String("activity")).firstChildElement(QLatin1String("meta-data"));
     while (!metadataElem.isNull()) {
         if (metadataElem.attribute(QLatin1String("android:name")) == QLatin1String("android.app.lib_name")) {
             m_targetLineEdit->setEditText(metadataElem.attribute(QLatin1String("android:value")));
@@ -806,10 +793,10 @@ void AndroidManifestEditorWidget::syncToWidgets(const QDomDocument &doc)
     m_mIconPath.clear();
     m_hIconPath.clear();
 
-    disconnect(m_defaultPermissonsCheckBox, SIGNAL(stateChanged(int)),
-            this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
-    disconnect(m_defaultFeaturesCheckBox, SIGNAL(stateChanged(int)),
-            this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
+    disconnect(m_defaultPermissonsCheckBox, &QCheckBox::stateChanged,
+            this, &AndroidManifestEditorWidget::defaultPermissionOrFeatureCheckBoxClicked);
+    disconnect(m_defaultFeaturesCheckBox, &QCheckBox::stateChanged,
+            this, &AndroidManifestEditorWidget::defaultPermissionOrFeatureCheckBoxClicked);
 
     m_defaultPermissonsCheckBox->setChecked(false);
     m_defaultFeaturesCheckBox->setChecked(false);
@@ -830,10 +817,10 @@ void AndroidManifestEditorWidget::syncToWidgets(const QDomDocument &doc)
     m_defaultPermissonsCheckBox->setChecked(foundPermissionComment);
     m_defaultFeaturesCheckBox->setChecked(foundFeatureComment);
 
-    connect(m_defaultPermissonsCheckBox, SIGNAL(stateChanged(int)),
-            this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
-    connect(m_defaultFeaturesCheckBox, SIGNAL(stateChanged(int)),
-            this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
+    connect(m_defaultPermissonsCheckBox, &QCheckBox::stateChanged,
+            this, &AndroidManifestEditorWidget::defaultPermissionOrFeatureCheckBoxClicked);
+    connect(m_defaultFeaturesCheckBox, &QCheckBox::stateChanged,
+            this, &AndroidManifestEditorWidget::defaultPermissionOrFeatureCheckBoxClicked);
 
     QStringList permissions;
     QDomElement permissionElem = manifest.firstChildElement(QLatin1String("uses-permission"));
@@ -894,7 +881,7 @@ void AndroidManifestEditorWidget::syncToEditor()
 
 namespace {
 QXmlStreamAttributes modifyXmlStreamAttributes(const QXmlStreamAttributes &input, const QStringList &keys,
-                                               const QStringList values, const QStringList &remove = QStringList())
+                                               const QStringList &values, const QStringList &remove = QStringList())
 {
     Q_ASSERT(keys.size() == values.size());
     QXmlStreamAttributes result;
@@ -931,7 +918,7 @@ void AndroidManifestEditorWidget::parseManifest(QXmlStreamReader &reader, QXmlSt
             << QLatin1String("android:versionName");
     QStringList values = QStringList()
             << m_packageNameLineEdit->text()
-            << QString::number(m_versionCode->value())
+            << m_versionCodeLineEdit->text()
             << m_versionNameLinedit->text();
 
     QXmlStreamAttributes result = modifyXmlStreamAttributes(attributes, keys, values);
@@ -1004,13 +991,8 @@ void AndroidManifestEditorWidget::parseApplication(QXmlStreamReader &reader, QXm
     writer.writeStartElement(reader.name().toString());
 
     QXmlStreamAttributes attributes = reader.attributes();
-    QStringList keys;
-    QStringList values;
-    if (!m_appNameInStringsXml) {
-        keys << QLatin1String("android:label");
-        values << m_appNameLineEdit->text();
-        m_setAppName = false;
-    }
+    QStringList keys = {QLatin1String("android:label")};
+    QStringList values = {m_appNameLineEdit->text()};
     bool ensureIconAttribute =  !m_lIconPath.isEmpty()
             || !m_mIconPath.isEmpty()
             || !m_hIconPath.isEmpty();
@@ -1044,7 +1026,14 @@ void AndroidManifestEditorWidget::parseApplication(QXmlStreamReader &reader, QXm
 void AndroidManifestEditorWidget::parseActivity(QXmlStreamReader &reader, QXmlStreamWriter &writer)
 {
     Q_ASSERT(reader.isStartElement());
-    writer.writeCurrentToken(reader);
+
+    writer.writeStartElement(reader.name().toString());
+    QXmlStreamAttributes attributes = reader.attributes();
+    QStringList keys = { QLatin1String("android:label") };
+    QStringList values = { m_activityNameLineEdit->text() };
+    QXmlStreamAttributes result = modifyXmlStreamAttributes(attributes, keys, values);
+    writer.writeAttributes(result);
+
     reader.readNext();
 
     bool found = false;
@@ -1081,8 +1070,8 @@ bool AndroidManifestEditorWidget::parseMetaData(QXmlStreamReader &reader, QXmlSt
     QXmlStreamAttributes result;
 
     if (attributes.value(QLatin1String("android:name")) == QLatin1String("android.app.lib_name")) {
-        QStringList keys = QStringList() << QLatin1String("android:value");
-        QStringList values = QStringList() << m_targetLineEdit->currentText();
+        QStringList keys = QStringList("android:value");
+        QStringList values = QStringList(m_targetLineEdit->currentText());
         result = modifyXmlStreamAttributes(attributes, keys, values);
         found = true;
     } else {
@@ -1158,7 +1147,9 @@ void AndroidManifestEditorWidget::parseUsesSdk(QXmlStreamReader &reader, QXmlStr
     }
 }
 
-QString AndroidManifestEditorWidget::parseUsesPermission(QXmlStreamReader &reader, QXmlStreamWriter &writer, const QSet<QString> permissions)
+QString AndroidManifestEditorWidget::parseUsesPermission(QXmlStreamReader &reader,
+                                                         QXmlStreamWriter &writer,
+                                                         const QSet<QString> &permissions)
 {
     Q_ASSERT(reader.isStartElement());
 
@@ -1328,12 +1319,6 @@ void AndroidManifestEditorWidget::removePermission()
     setDirty(true);
 }
 
-void AndroidManifestEditorWidget::setAppName()
-{
-    m_setAppName = true;
-    setDirty(true);
-}
-
 void AndroidManifestEditorWidget::setPackageName()
 {
     const QString packageName= m_packageNameLineEdit->text();
@@ -1367,21 +1352,23 @@ const QStringList &PermissionsModel::permissions()
 
 QModelIndex PermissionsModel::addPermission(const QString &permission)
 {
-    const int idx = qLowerBound(m_permissions, permission) - m_permissions.constBegin();
+    auto it = std::lower_bound(m_permissions.constBegin(), m_permissions.constEnd(), permission);
+    const int idx = it - m_permissions.constBegin();
     beginInsertRows(QModelIndex(), idx, idx);
     m_permissions.insert(idx, permission);
     endInsertRows();
     return index(idx);
 }
 
-bool PermissionsModel::updatePermission(QModelIndex index, const QString &permission)
+bool PermissionsModel::updatePermission(const QModelIndex &index, const QString &permission)
 {
     if (!index.isValid())
         return false;
     if (m_permissions[index.row()] == permission)
         return false;
 
-    int newIndex = qLowerBound(m_permissions.constBegin(), m_permissions.constEnd(), permission) - m_permissions.constBegin();
+    auto it = std::lower_bound(m_permissions.constBegin(), m_permissions.constEnd(), permission);
+    const int newIndex = it - m_permissions.constBegin();
     if (newIndex == index.row() || newIndex == index.row() + 1) {
         m_permissions[index.row()] = permission;
         emit dataChanged(index, index);
@@ -1429,7 +1416,20 @@ AndroidManifestTextEditorWidget::AndroidManifestTextEditorWidget(AndroidManifest
     : TextEditor::TextEditorWidget(parent)
 {
     setTextDocument(TextEditor::TextDocumentPtr(new AndroidManifestDocument(parent)));
-    setupAsPlainEditor();
     textDocument()->setMimeType(QLatin1String(Constants::ANDROID_MANIFEST_MIME_TYPE));
+    setupGenericHighlighter();
+    setMarksVisible(false);
+
+    // this context is used by the TextEditorActionHandler registered for the text editor in
+    // the AndroidManifestEditorFactory
+    m_context = new Core::IContext(this);
+    m_context->setWidget(this);
+    m_context->setContext(Core::Context(Constants::ANDROID_MANIFEST_EDITOR_CONTEXT));
+    Core::ICore::addContextObject(m_context);
+}
+
+AndroidManifestTextEditorWidget::~AndroidManifestTextEditorWidget()
+{
+    Core::ICore::removeContextObject(m_context);
 }
 
