@@ -27,7 +27,6 @@
 
 #include <projectexplorer/devicesupport/deviceprocess.h>
 
-namespace ProjectExplorer { class Runnable; }
 namespace Utils { class QtcProcess; }
 
 namespace BareMetal {
@@ -39,7 +38,7 @@ class GdbServerProviderProcess : public ProjectExplorer::DeviceProcess
 public:
     explicit GdbServerProviderProcess(
             const QSharedPointer<const ProjectExplorer::IDevice> &device,
-            QObject *parent = 0);
+            QObject *parent = nullptr);
 
     void start(const ProjectExplorer::Runnable &runnable) override;
     void interrupt() override;
