@@ -26,7 +26,9 @@
 #include "clangeditordocumentprocessor.h"
 #include "clangfollowsymbol.h"
 
+#include <coreplugin/editormanager/editormanager.h>
 #include <cpptools/cppmodelmanager.h>
+#include <cpptools/cppfollowsymbolundercursor.h>
 #include <texteditor/texteditor.h>
 
 #include <clangsupport/tokeninfocontainer.h>
@@ -34,6 +36,8 @@
 #include <utils/textutils.h>
 #include <utils/algorithm.h>
 
+#include <memory>
+
 namespace ClangCodeModel {
 namespace Internal {
 
@@ -155,6 +159,12 @@ static ::Utils::ProcessLinkCallback extendedCallback(::Utils::ProcessLinkCallbac
     };
 }
 
+static bool isSameInvocationContext(const Utils::FileName &filePath)
+{
+    return TextEditor::BaseTextEditor::currentTextEditor()->editorWidget()->isVisible()
+        && Core::EditorManager::currentDocument()->filePath() == filePath;
+}
+
 void ClangFollowSymbol::findLink(const CppTools::CursorInEditor &data,
                                  ::Utils::ProcessLinkCallback &&processLinkCallback,
                                  bool resolveTarget,
@@ -174,12 +184,22 @@ void ClangFollowSymbol::findLink(const CppTools::CursorInEditor &data,
         return processLinkCallback(Utils::Link());
 
     if (!resolveTarget) {
-        processLinkCallback(linkAtCursor(cursor,
-                                         data.filePath().toString(),
-                                         static_cast<uint>(line),
-                                         static_cast<uint>(column),
-                                         processor));
-        return;
+        Utils::Link link = linkAtCursor(cursor,
+                                        data.filePath().toString(),
+                                        static_cast<uint>(line),
+                                        static_cast<uint>(column),
+                                        processor);
+        if (link == Utils::Link()) {
+            CppTools::FollowSymbolUnderCursor followSymbol;
+            return followSymbol.findLink(data,
+                                         std::move(processLinkCallback),
+                                         false,
+                                         snapshot,
+                                         documentFromSemanticInfo,
+                                         symbolFinder,
+                                         inNextSplit);
+        }
+        return processLinkCallback(link);
     }
 
     QFuture<CppTools::SymbolInfo> infoFuture
@@ -192,11 +212,11 @@ void ClangFollowSymbol::findLink(const CppTools::CursorInEditor &data,
     if (m_watcher)
         m_watcher->cancel();
 
-    m_watcher.reset(new FutureSymbolWatcher());
+    m_watcher = std::make_unique<FutureSymbolWatcher>();
 
-    QObject::connect(m_watcher.get(), &FutureSymbolWatcher::finished,
-                     [=, callback=std::move(processLinkCallback)]() mutable {
-        if (m_watcher->isCanceled())
+    QObject::connect(m_watcher.get(), &FutureSymbolWatcher::finished, [=, filePath=data.filePath(),
+                     callback=std::move(processLinkCallback)]() mutable {
+        if (m_watcher->isCanceled() || !isSameInvocationContext(filePath))
             return callback(Utils::Link());
         CppTools::SymbolInfo result = m_watcher->result();
         // We did not fail but the result is empty
