@@ -34,6 +34,7 @@
 
 #include <QAction>
 #include <QScrollBar>
+#include <QTextBlock>
 
 using namespace Utils;
 
@@ -64,7 +65,7 @@ public:
     Qt::MouseButton mouseButtonPressed = Qt::NoButton;
     bool m_zoomEnabled = false;
     float m_originalFontSize = 0.;
-    int maxLineCount = 100000;
+    int maxCharCount = Core::Constants::DEFAULT_MAX_CHAR_COUNT;
     QTextCursor cursor;
 };
 
@@ -87,12 +88,12 @@ OutputWindow::OutputWindow(Context context, QWidget *parent)
     d->outputWindowContext->setWidget(this);
     ICore::addContextObject(d->outputWindowContext);
 
-    QAction *undoAction = new QAction(this);
-    QAction *redoAction = new QAction(this);
-    QAction *cutAction = new QAction(this);
-    QAction *copyAction = new QAction(this);
-    QAction *pasteAction = new QAction(this);
-    QAction *selectAllAction = new QAction(this);
+    auto undoAction = new QAction(this);
+    auto redoAction = new QAction(this);
+    auto cutAction = new QAction(this);
+    auto copyAction = new QAction(this);
+    auto pasteAction = new QAction(this);
+    auto selectAllAction = new QAction(this);
 
     ActionManager::registerAction(undoAction, Constants::UNDO, context);
     ActionManager::registerAction(redoAction, Constants::REDO, context);
@@ -268,21 +269,42 @@ QString OutputWindow::doNewlineEnforcement(const QString &out)
     return s;
 }
 
-void OutputWindow::setMaxLineCount(int count)
+void OutputWindow::setMaxCharCount(int count)
 {
-    d->maxLineCount = count;
-    setMaximumBlockCount(d->maxLineCount);
+    d->maxCharCount = count;
+    setMaximumBlockCount(count / 100);
 }
 
-int OutputWindow::maxLineCount() const
+int OutputWindow::maxCharCount() const
 {
-    return d->maxLineCount;
+    return d->maxCharCount;
 }
 
 void OutputWindow::appendMessage(const QString &output, OutputFormat format)
 {
-    const QString out = SynchronousProcess::normalizeNewlines(output);
-    setMaximumBlockCount(d->maxLineCount);
+    QString out = SynchronousProcess::normalizeNewlines(output);
+
+    if (out.size() > d->maxCharCount) {
+        // Current line alone exceeds limit, we need to cut it.
+        out.truncate(d->maxCharCount);
+        out.append("[...]");
+        setMaximumBlockCount(1);
+    } else {
+        int plannedChars = document()->characterCount() + out.size();
+        if (plannedChars > d->maxCharCount) {
+            int plannedBlockCount = document()->blockCount();
+            QTextBlock tb = document()->firstBlock();
+            while (tb.isValid() && plannedChars > d->maxCharCount && plannedBlockCount > 1) {
+                plannedChars -= tb.length();
+                plannedBlockCount -= 1;
+                tb = tb.next();
+            }
+            setMaximumBlockCount(plannedBlockCount);
+        } else {
+            setMaximumBlockCount(-1);
+        }
+    }
+
     const bool atBottom = isScrollbarAtBottom() || m_scrollTimer.isActive();
 
     if (format == ErrorMessageFormat || format == NormalMessageFormat) {
@@ -300,7 +322,9 @@ void OutputWindow::appendMessage(const QString &output, OutputFormat format)
             bool enforceNewline = d->enforceNewline;
             d->enforceNewline = false;
 
-            if (!enforceNewline) {
+            if (enforceNewline) {
+                out.prepend('\n');
+            } else {
                 newline = out.indexOf(QLatin1Char('\n'));
                 moveCursor(QTextCursor::End);
                 if (newline != -1 && d->formatter)
@@ -316,7 +340,7 @@ void OutputWindow::appendMessage(const QString &output, OutputFormat format)
                     s.chop(1);
                 }
                 if (d->formatter)
-                    d->formatter->appendMessage(QLatin1Char('\n') + s, format);
+                    d->formatter->appendMessage(s, format);
             }
         } else {
             if (d->formatter)
@@ -341,7 +365,7 @@ void OutputWindow::appendMessage(const QString &output, OutputFormat format)
 void OutputWindow::appendText(const QString &textIn, const QTextCharFormat &format)
 {
     const QString text = SynchronousProcess::normalizeNewlines(textIn);
-    if (d->maxLineCount > 0 && document()->blockCount() >= d->maxLineCount)
+    if (d->maxCharCount > 0 && document()->characterCount() >= d->maxCharCount)
         return;
     const bool atBottom = isScrollbarAtBottom();
     if (!d->cursor.atEnd())
@@ -349,10 +373,12 @@ void OutputWindow::appendText(const QString &textIn, const QTextCharFormat &form
     d->cursor.beginEditBlock();
     d->cursor.insertText(doNewlineEnforcement(text), format);
 
-    if (d->maxLineCount > 0 && document()->blockCount() >= d->maxLineCount) {
+    if (d->maxCharCount > 0 && document()->characterCount() >= d->maxCharCount) {
         QTextCharFormat tmp;
         tmp.setFontWeight(QFont::Bold);
-        d->cursor.insertText(doNewlineEnforcement(tr("Additional output omitted") + QLatin1Char('\n')), tmp);
+        d->cursor.insertText(doNewlineEnforcement(tr("Additional output omitted. You can increase "
+                                                     "the limit in the \"Build & Run\" settings.")
+                                                  + QLatin1Char('\n')), tmp);
     }
 
     d->cursor.endEditBlock();
