@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef CPPEDITORCONSTANTS_H
-#define CPPEDITORCONSTANTS_H
+#pragma once
 
 namespace CppEditor {
 namespace Constants {
@@ -35,8 +34,10 @@ const char CPPEDITOR_DISPLAY_NAME[] = QT_TRANSLATE_NOOP("OpenWith::Editors", "C+
 const char SWITCH_DECLARATION_DEFINITION[] = "CppEditor.SwitchDeclarationDefinition";
 const char OPEN_DECLARATION_DEFINITION_IN_NEXT_SPLIT[] = "CppEditor.OpenDeclarationDefinitionInNextSplit";
 const char RENAME_SYMBOL_UNDER_CURSOR[] = "CppEditor.RenameSymbolUnderCursor";
-const char FIND_USAGES[] = "CppEditor.FindUsages";
 const char OPEN_PREPROCESSOR_DIALOG[] = "CppEditor.OpenPreprocessorDialog";
+const char ERRORS_IN_HEADER_FILES[] = "CppEditor.ErrorsInHeaderFiles";
+const char MULTIPLE_PARSE_CONTEXTS_AVAILABLE[] = "CppEditor.MultipleParseContextsAvailable";
+const char NO_PROJECT_CONFIGURATION[] = "CppEditor.NoProjectConfiguration";
 const char M_REFACTORING_MENU_INSERTION_POINT[] = "CppEditor.RefactorGroup";
 const char UPDATE_CODEMODEL[] = "CppEditor.UpdateCodeModel";
 const char INSPECT_CPP_CODEMODEL[] = "CppEditor.InspectCppCodeModel";
@@ -49,9 +50,8 @@ const char OPEN_INCLUDE_HIERARCHY[] = "CppEditor.OpenIncludeHierarchy";
 
 const char CPP_SNIPPETS_GROUP_ID[] = "C++";
 
-const char CPP_PREPROCESSOR_PROJECT_PREFIX[] = "CppPreprocessorProject-";
+const char EXTRA_PREPROCESSOR_DIRECTIVES[] = "CppEditor.ExtraPreprocessorDirectives-";
+const char PREFERRED_PARSE_CONTEXT[] = "CppEditor.PreferredParseContext-";
 
 } // namespace Constants
 } // namespace CppEditor
-
-#endif // CPPEDITORCONSTANTS_H
