@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -33,36 +28,21 @@
 
 #include <cpptools/cppdoxygen.h>
 #include <cpptools/cpptoolsreuse.h>
-#include <texteditor/basetextdocumentlayout.h>
+#include <texteditor/textdocumentlayout.h>
 
 #include <cplusplus/SimpleLexer.h>
 #include <cplusplus/Lexer.h>
 
 #include <QTextDocument>
 
-using namespace CppEditor::Internal;
+using namespace CppEditor;
 using namespace TextEditor;
 using namespace CPlusPlus;
 
 CppHighlighter::CppHighlighter(QTextDocument *document) :
-    TextEditor::SyntaxHighlighter(document)
+    SyntaxHighlighter(document)
 {
-    static QVector<TextEditor::TextStyle> categories;
-    if (categories.isEmpty()) {
-        categories << TextEditor::C_NUMBER
-                   << TextEditor::C_STRING
-                   << TextEditor::C_TYPE
-                   << TextEditor::C_KEYWORD
-                   << TextEditor::C_PRIMITIVE_TYPE
-                   << TextEditor::C_OPERATOR
-                   << TextEditor::C_PREPROCESSOR
-                   << TextEditor::C_LABEL
-                   << TextEditor::C_COMMENT
-                   << TextEditor::C_DOXYGEN_COMMENT
-                   << TextEditor::C_DOXYGEN_TAG
-                   << TextEditor::C_VISUAL_WHITESPACE;
-    }
-    setTextFormatCategories(categories);
+    setDefaultTextFormatCategories();
 }
 
 void CppHighlighter::highlightBlock(const QString &text)
@@ -76,21 +56,16 @@ void CppHighlighter::highlightBlock(const QString &text)
 
     int braceDepth = initialBraceDepth;
 
-    // FIXME: Check defaults or get from document.
-    LanguageFeatures features;
-    features.cxx11Enabled = true;
-    features.c99Enabled = true;
-
     SimpleLexer tokenize;
-    tokenize.setLanguageFeatures(features);
+    tokenize.setLanguageFeatures(m_languageFeatures);
 
     int initialLexerState = lexerState;
-    const QList<Token> tokens = tokenize(text, initialLexerState);
+    const Tokens tokens = tokenize(text, initialLexerState);
     lexerState = tokenize.state(); // refresh lexer state
 
     initialLexerState &= ~0x80; // discard newline expected bit
     int foldingIndent = initialBraceDepth;
-    if (TextBlockUserData *userData = BaseTextDocumentLayout::testUserData(currentBlock())) {
+    if (TextBlockUserData *userData = TextDocumentLayout::testUserData(currentBlock())) {
         userData->setFoldingIndent(0);
         userData->setFoldingStartIncluded(false);
         userData->setFoldingEndIncluded(false);
@@ -98,23 +73,23 @@ void CppHighlighter::highlightBlock(const QString &text)
 
     if (tokens.isEmpty()) {
         setCurrentBlockState((braceDepth << 8) | lexerState);
-        BaseTextDocumentLayout::clearParentheses(currentBlock());
+        TextDocumentLayout::clearParentheses(currentBlock());
         if (text.length())  {// the empty line can still contain whitespace
             if (initialLexerState == T_COMMENT)
-                highlightLine(text, 0, text.length(), formatForCategory(CppCommentFormat));
+                setFormatWithSpaces(text, 0, text.length(), formatForCategory(C_COMMENT));
             else if (initialLexerState == T_DOXY_COMMENT)
-                highlightLine(text, 0, text.length(), formatForCategory(CppDoxygenCommentFormat));
+                setFormatWithSpaces(text, 0, text.length(), formatForCategory(C_DOXYGEN_COMMENT));
             else
-                setFormat(0, text.length(), formatForCategory(CppVisualWhitespace));
+                setFormat(0, text.length(), formatForCategory(C_VISUAL_WHITESPACE));
         }
-        BaseTextDocumentLayout::setFoldingIndent(currentBlock(), foldingIndent);
+        TextDocumentLayout::setFoldingIndent(currentBlock(), foldingIndent);
         return;
     }
 
     const unsigned firstNonSpace = tokens.first().utf16charsBegin();
 
     Parentheses parentheses;
-    parentheses.reserve(20); // assume wizard level ;-)
+    parentheses.reserve(5);
 
     bool expectPreprocessorKeyword = false;
     bool onlyHighlightComments = false;
@@ -132,7 +107,7 @@ void CppHighlighter::highlightBlock(const QString &text)
         if (previousTokenEnd != tk.utf16charsBegin()) {
             setFormat(previousTokenEnd,
                       tk.utf16charsBegin() - previousTokenEnd,
-                      formatForCategory(CppVisualWhitespace));
+                      formatForCategory(C_VISUAL_WHITESPACE));
         }
 
         if (tk.is(T_LPAREN) || tk.is(T_LBRACE) || tk.is(T_LBRACKET)) {
@@ -145,7 +120,7 @@ void CppHighlighter::highlightBlock(const QString &text)
                 // as if it were inside the folding block
                 if (tk.utf16charsBegin() == firstNonSpace) {
                     ++foldingIndent;
-                    BaseTextDocumentLayout::userData(currentBlock())->setFoldingStartIncluded(true);
+                    TextDocumentLayout::userData(currentBlock())->setFoldingStartIncluded(true);
                 }
             }
         } else if (tk.is(T_RPAREN) || tk.is(T_RBRACE) || tk.is(T_RBRACKET)) {
@@ -156,7 +131,7 @@ void CppHighlighter::highlightBlock(const QString &text)
                 if (braceDepth < foldingIndent) {
                     // unless we are at the end of the block, we reduce the folding indent
                     if (i == tokens.size()-1 || tokens.at(i+1).is(T_SEMICOLON))
-                        BaseTextDocumentLayout::userData(currentBlock())->setFoldingEndIncluded(true);
+                        TextDocumentLayout::userData(currentBlock())->setFoldingEndIncluded(true);
                     else
                         foldingIndent = qMin(braceDepth, foldingIndent);
                 }
@@ -172,13 +147,13 @@ void CppHighlighter::highlightBlock(const QString &text)
             continue;
 
         if (i == 0 && tk.is(T_POUND)) {
-            highlightLine(text, tk.utf16charsBegin(), tk.utf16chars(),
-                          formatForCategory(CppPreprocessorFormat));
+            setFormatWithSpaces(text, tk.utf16charsBegin(), tk.utf16chars(),
+                          formatForCategory(C_PREPROCESSOR));
             expectPreprocessorKeyword = true;
         } else if (highlightCurrentWordAsPreprocessor
                    && (tk.isKeyword() || tk.is(T_IDENTIFIER))
                    && isPPKeyword(text.midRef(tk.utf16charsBegin(), tk.utf16chars()))) {
-            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(CppPreprocessorFormat));
+            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(C_PREPROCESSOR));
             const QStringRef ppKeyword = text.midRef(tk.utf16charsBegin(), tk.utf16chars());
             if (ppKeyword == QLatin1String("error")
                     || ppKeyword == QLatin1String("warning")
@@ -187,14 +162,14 @@ void CppHighlighter::highlightBlock(const QString &text)
             }
 
         } else if (tk.is(T_NUMERIC_LITERAL)) {
-            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(CppNumberFormat));
+            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(C_NUMBER));
         } else if (tk.isStringLiteral() || tk.isCharLiteral()) {
-            highlightLine(text, tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(CppStringFormat));
+            setFormatWithSpaces(text, tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(C_STRING));
         } else if (tk.isComment()) {
             const int startPosition = initialLexerState ? previousTokenEnd : tk.utf16charsBegin();
             if (tk.is(T_COMMENT) || tk.is(T_CPP_COMMENT)) {
-                highlightLine(text, startPosition, tk.utf16charsEnd() - startPosition,
-                              formatForCategory(CppCommentFormat));
+                setFormatWithSpaces(text, startPosition, tk.utf16charsEnd() - startPosition,
+                              formatForCategory(C_COMMENT));
             }
 
             else // a doxygen comment
@@ -208,7 +183,7 @@ void CppHighlighter::highlightBlock(const QString &text)
                 --braceDepth;
                 // unless we are at the end of the block, we reduce the folding indent
                 if (i == tokens.size()-1)
-                    BaseTextDocumentLayout::userData(currentBlock())->setFoldingEndIncluded(true);
+                    TextDocumentLayout::userData(currentBlock())->setFoldingEndIncluded(true);
                 else
                     foldingIndent = qMin(braceDepth, foldingIndent);
                 const int tokenEnd = tk.utf16charsBegin() + tk.utf16chars() - 1;
@@ -219,16 +194,19 @@ void CppHighlighter::highlightBlock(const QString &text)
             }
 
         } else if (tk.isKeyword()
-                   || CppTools::isQtKeyword(text.midRef(tk.utf16charsBegin(), tk.utf16chars()))
-                   || tk.isObjCAtKeyword()) {
-            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(CppKeywordFormat));
+                   || (m_languageFeatures.qtKeywordsEnabled
+                       && CppTools::isQtKeyword(text.midRef(tk.utf16charsBegin(), tk.utf16chars())))
+                   || (m_languageFeatures.objCEnabled && tk.isObjCAtKeyword())) {
+            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(C_KEYWORD));
         } else if (tk.isPrimitiveType()) {
             setFormat(tk.utf16charsBegin(), tk.utf16chars(),
-                      formatForCategory(CppPrimitiveTypeFormat));
+                      formatForCategory(C_PRIMITIVE_TYPE));
         } else if (tk.isOperator()) {
-            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(CppOperatorFormat));
+            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(C_OPERATOR));
+        } else if (tk.isPunctuation()) {
+            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(C_PUNCTUATION));
         } else if (i == 0 && tokens.size() > 1 && tk.is(T_IDENTIFIER) && tokens.at(1).is(T_COLON)) {
-            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(CppLabelFormat));
+            setFormat(tk.utf16charsBegin(), tk.utf16chars(), formatForCategory(C_LABEL));
         } else if (tk.is(T_IDENTIFIER)) {
             highlightWord(text.midRef(tk.utf16charsBegin(), tk.utf16chars()), tk.utf16charsBegin(),
                           tk.utf16chars());
@@ -238,7 +216,7 @@ void CppHighlighter::highlightBlock(const QString &text)
     // mark the trailing white spaces
     const int lastTokenEnd = tokens.last().utf16charsEnd();
     if (text.length() > lastTokenEnd)
-        highlightLine(text, lastTokenEnd, text.length() - lastTokenEnd, formatForCategory(CppVisualWhitespace));
+        formatSpaces(text, lastTokenEnd, text.length() - lastTokenEnd);
 
     if (!initialLexerState && lexerState && !tokens.isEmpty()) {
         const Token &lastToken = tokens.last();
@@ -249,17 +227,17 @@ void CppHighlighter::highlightBlock(const QString &text)
         }
     }
 
-    BaseTextDocumentLayout::setParentheses(currentBlock(), parentheses);
+    TextDocumentLayout::setParentheses(currentBlock(), parentheses);
 
     // if the block is ifdefed out, we only store the parentheses, but
 
     // do not adjust the brace depth.
-    if (BaseTextDocumentLayout::ifdefedOut(currentBlock())) {
+    if (TextDocumentLayout::ifdefedOut(currentBlock())) {
         braceDepth = initialBraceDepth;
         foldingIndent = initialBraceDepth;
     }
 
-    BaseTextDocumentLayout::setFoldingIndent(currentBlock(), foldingIndent);
+    TextDocumentLayout::setFoldingIndent(currentBlock(), foldingIndent);
 
     // optimization: if only the brace depth changes, we adjust subsequent blocks
     // to have QSyntaxHighlighter stop the rehighlighting
@@ -268,13 +246,13 @@ void CppHighlighter::highlightBlock(const QString &text)
         int oldState = currentState & 0xff;
         int oldBraceDepth = currentState >> 8;
         if (oldState == tokenize.state() && oldBraceDepth != braceDepth) {
-            BaseTextDocumentLayout::FoldValidator foldValidor;
-            foldValidor.setup(qobject_cast<BaseTextDocumentLayout *>(document()->documentLayout()));
+            TextDocumentLayout::FoldValidator foldValidor;
+            foldValidor.setup(qobject_cast<TextDocumentLayout *>(document()->documentLayout()));
             int delta = braceDepth - oldBraceDepth;
             QTextBlock block = currentBlock().next();
             while (block.isValid() && block.userState() != -1) {
-                BaseTextDocumentLayout::changeBraceDepth(block, delta);
-                BaseTextDocumentLayout::changeFoldingIndent(block, delta);
+                TextDocumentLayout::changeBraceDepth(block, delta);
+                TextDocumentLayout::changeFoldingIndent(block, delta);
                 foldValidor.process(block);
                 block = block.next();
             }
@@ -285,6 +263,13 @@ void CppHighlighter::highlightBlock(const QString &text)
     setCurrentBlockState((braceDepth << 8) | tokenize.state());
 }
 
+void CppHighlighter::setLanguageFeatures(const LanguageFeatures &languageFeatures)
+{
+    if (languageFeatures != m_languageFeatures) {
+        m_languageFeatures = languageFeatures;
+        rehighlight();
+    }
+}
 
 bool CppHighlighter::isPPKeyword(const QStringRef &text) const
 {
@@ -360,30 +345,6 @@ bool CppHighlighter::isPPKeyword(const QStringRef &text) const
     return false;
 }
 
-void CppHighlighter::highlightLine(const QString &text, int position, int length,
-                                   const QTextCharFormat &format)
-{
-    QTextCharFormat visualSpaceFormat = formatForCategory(CppVisualWhitespace);
-    visualSpaceFormat.setBackground(format.background());
-
-    const int end = position + length;
-    int index = position;
-
-    while (index != end) {
-        const bool isSpace = text.at(index).isSpace();
-        const int start = index;
-
-        do { ++index; }
-        while (index != end && text.at(index).isSpace() == isSpace);
-
-        const int tokenLength = index - start;
-        if (isSpace)
-            setFormat(start, tokenLength, visualSpaceFormat);
-        else if (format.isValid())
-            setFormat(start, tokenLength, format);
-    }
-}
-
 void CppHighlighter::highlightWord(QStringRef word, int position, int length)
 {
     // try to highlight Qt 'identifiers' like QObject and Q_PROPERTY
@@ -397,7 +358,7 @@ void CppHighlighter::highlightWord(QStringRef word, int position, int length)
                     return;
             }
 
-            setFormat(position, length, formatForCategory(CppTypeFormat));
+            setFormat(position, length, formatForCategory(C_TYPE));
         }
     }
 }
@@ -409,8 +370,8 @@ void CppHighlighter::highlightDoxygenComment(const QString &text, int position,
     const QChar *uc = text.unicode();
     const QChar *it = uc + position;
 
-    const QTextCharFormat &format = formatForCategory(CppDoxygenCommentFormat);
-    const QTextCharFormat &kwFormat = formatForCategory(CppDoxygenTagFormat);
+    const QTextCharFormat &format = formatForCategory(C_DOXYGEN_COMMENT);
+    const QTextCharFormat &kwFormat = formatForCategory(C_DOXYGEN_TAG);
 
     while (!it->isNull()) {
         if (it->unicode() == QLatin1Char('\\') ||
@@ -423,7 +384,7 @@ void CppHighlighter::highlightDoxygenComment(const QString &text, int position,
 
             int k = CppTools::classifyDoxygenTag(start, it - start);
             if (k != CppTools::T_DOXY_IDENTIFIER) {
-                highlightLine(text, initial, start - uc - initial, format);
+                setFormatWithSpaces(text, initial, start - uc - initial, format);
                 setFormat(start - uc - 1, it - start + 1, kwFormat);
                 initial = it - uc;
             }
@@ -431,6 +392,6 @@ void CppHighlighter::highlightDoxygenComment(const QString &text, int position,
             ++it;
     }
 
-    highlightLine(text, initial, it - uc - initial, format);
+    setFormatWithSpaces(text, initial, it - uc - initial, format);
 }
 
