@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -37,6 +32,7 @@
 #include "cppsourceprocessor.h"
 #include "cpptoolsconstants.h"
 #include "cpptoolsplugin.h"
+#include "cpptoolsreuse.h"
 #include "searchsymbols.h"
 
 #include <coreplugin/icore.h>
@@ -46,10 +42,11 @@
 #include <cplusplus/LookupContext.h>
 #include <utils/qtcassert.h>
 #include <utils/runextensions.h>
+#include <utils/temporarydirectory.h>
 
 #include <QCoreApplication>
-#include <QDir>
 #include <QElapsedTimer>
+#include <QRegularExpression>
 
 using namespace CppTools;
 using namespace CppTools::Internal;
@@ -61,9 +58,10 @@ namespace {
 class ParseParams
 {
 public:
-    ProjectPart::HeaderPaths headerPaths;
+    ProjectExplorer::HeaderPaths headerPaths;
     WorkingCopy workingCopy;
     QSet<QString> sourceFiles;
+    int indexerFileSizeLimitInMb = -1;
 };
 
 class WriteTaskFileForDiagnostics
@@ -72,12 +70,10 @@ class WriteTaskFileForDiagnostics
 
 public:
     WriteTaskFileForDiagnostics()
-        : m_processedDiagnostics(0)
     {
-        const QString fileName = QDir::tempPath()
-                + QLatin1String("/qtc_findErrorsIndexing.diagnostics.")
-                + QDateTime::currentDateTime().toString(QLatin1String("yyMMdd_HHmm"))
-                + QLatin1String(".tasks");
+        const QString fileName = Utils::TemporaryDirectory::masterDirectoryPath()
+                + "/qtc_findErrorsIndexing.diagnostics."
+                + QDateTime::currentDateTime().toString("yyMMdd_HHmm") + ".tasks";
 
         m_file.setFileName(fileName);
         Q_ASSERT(m_file.open(QIODevice::WriteOnly | QIODevice::Text));
@@ -123,7 +119,7 @@ public:
 private:
     QFile m_file;
     QTextStream m_out;
-    int m_processedDiagnostics;
+    int m_processedDiagnostics = 0;
 };
 
 void classifyFiles(const QSet<QString> &files, QStringList *headers, QStringList *sources)
@@ -136,7 +132,9 @@ void classifyFiles(const QSet<QString> &files, QStringList *headers, QStringList
     }
 }
 
-void indexFindErrors(QFutureInterface<void> &future, const ParseParams params)
+void indexFindErrors(QFutureInterface<void> &indexingFuture,
+                     const QFutureInterface<void> &superFuture,
+                     const ParseParams params)
 {
     QStringList sources, headers;
     classifyFiles(params.sourceFiles, &headers, &sources);
@@ -149,9 +147,7 @@ void indexFindErrors(QFutureInterface<void> &future, const ParseParams params)
     timer.start();
 
     for (int i = 0, end = files.size(); i < end ; ++i) {
-        if (future.isPaused())
-            future.waitForResume();
-        if (future.isCanceled())
+        if (indexingFuture.isCanceled() || superFuture.isCanceled())
             break;
 
         const QString file = files.at(i);
@@ -160,7 +156,7 @@ void indexFindErrors(QFutureInterface<void> &future, const ParseParams params)
         // Parse the file as precisely as possible
         BuiltinEditorDocumentParser parser(file);
         parser.setReleaseSourceAndAST(false);
-        parser.update(BuiltinEditorDocumentParser::InMemoryInfo(false));
+        parser.update({CppModelManager::instance()->workingCopy(), nullptr, Language::Cxx, false});
         CPlusPlus::Document::Ptr document = parser.document();
         QTC_ASSERT(document, return);
 
@@ -173,7 +169,7 @@ void indexFindErrors(QFutureInterface<void> &future, const ParseParams params)
 
         document->releaseSourceAndAST();
 
-        future.setProgressValue(files.size() - (files.size() - (i + 1)));
+        indexingFuture.setProgressValue(files.size() - (files.size() - (i + 1)));
     }
 
     const QTime format = QTime(0, 0, 0, 0).addMSecs(timer.elapsed() + 500);
@@ -181,9 +177,12 @@ void indexFindErrors(QFutureInterface<void> &future, const ParseParams params)
     qDebug("FindErrorsIndexing: Finished after %s.", qPrintable(time));
 }
 
-void index(QFutureInterface<void> &future, const ParseParams params)
+void index(QFutureInterface<void> &indexingFuture,
+           const QFutureInterface<void> &superFuture,
+           const ParseParams params)
 {
     QScopedPointer<CppSourceProcessor> sourceProcessor(CppModelManager::createSourceProcessor());
+    sourceProcessor->setFileSizeLimitInMb(params.indexerFileSizeLimitInMb);
     sourceProcessor->setHeaderPaths(params.headerPaths);
     sourceProcessor->setWorkingCopy(params.workingCopy);
 
@@ -203,14 +202,11 @@ void index(QFutureInterface<void> &future, const ParseParams params)
     bool processingHeaders = false;
 
     CppModelManager *cmm = CppModelManager::instance();
-    const ProjectPart::HeaderPaths fallbackHeaderPaths = cmm->headerPaths();
+    const ProjectExplorer::HeaderPaths fallbackHeaderPaths = cmm->headerPaths();
     const CPlusPlus::LanguageFeatures defaultFeatures =
             CPlusPlus::LanguageFeatures::defaultFeatures();
     for (int i = 0; i < files.size(); ++i) {
-        if (future.isPaused())
-            future.waitForResume();
-
-        if (future.isCanceled())
+        if (indexingFuture.isCanceled() || superFuture.isCanceled())
             break;
 
         const QString fileName = files.at(i);
@@ -229,33 +225,35 @@ void index(QFutureInterface<void> &future, const ParseParams params)
             processingHeaders = true;
         }
 
-        ProjectPart::HeaderPaths headerPaths = parts.isEmpty()
+        ProjectExplorer::HeaderPaths headerPaths = parts.isEmpty()
                 ? fallbackHeaderPaths
                 : parts.first()->headerPaths;
         sourceProcessor->setHeaderPaths(headerPaths);
         sourceProcessor->run(fileName);
 
-        future.setProgressValue(files.size() - sourceProcessor->todo().size());
+        indexingFuture.setProgressValue(files.size() - sourceProcessor->todo().size());
 
         if (isSourceFile)
             sourceProcessor->resetEnvironment();
     }
 }
 
-void parse(QFutureInterface<void> &future, const ParseParams params)
+void parse(QFutureInterface<void> &indexingFuture,
+           const QFutureInterface<void> &superFuture,
+           const ParseParams params)
 {
     const QSet<QString> &files = params.sourceFiles;
     if (files.isEmpty())
         return;
 
-    future.setProgressRange(0, files.size());
+    indexingFuture.setProgressRange(0, files.size());
 
     if (FindErrorsIndexing)
-        indexFindErrors(future, params);
+        indexFindErrors(indexingFuture, superFuture, params);
     else
-        index(future, params);
+        index(indexingFuture, superFuture, params);
 
-    future.setProgressValue(files.size());
+    indexingFuture.setProgressValue(files.size());
     CppModelManager::instance()->finishedRefreshingSourceFiles(files);
 }
 
@@ -263,31 +261,32 @@ class BuiltinSymbolSearcher: public SymbolSearcher
 {
 public:
     BuiltinSymbolSearcher(const CPlusPlus::Snapshot &snapshot,
-                          Parameters parameters, QSet<QString> fileNames)
+                          const Parameters &parameters, const QSet<QString> &fileNames)
         : m_snapshot(snapshot)
         , m_parameters(parameters)
         , m_fileNames(fileNames)
     {}
 
-    ~BuiltinSymbolSearcher()
-    {}
+    ~BuiltinSymbolSearcher() override = default;
 
-    void runSearch(QFutureInterface<Core::SearchResultItem> &future)
+    void runSearch(QFutureInterface<Core::SearchResultItem> &future) override
     {
         future.setProgressRange(0, m_snapshot.size());
         future.setProgressValue(0);
         int progress = 0;
 
-        SearchSymbols search(CppToolsPlugin::stringTable());
+        SearchSymbols search;
         search.setSymbolsToSearchFor(m_parameters.types);
         CPlusPlus::Snapshot::const_iterator it = m_snapshot.begin();
 
         QString findString = (m_parameters.flags & Core::FindRegularExpression
-                              ? m_parameters.text : QRegExp::escape(m_parameters.text));
+                              ? m_parameters.text : QRegularExpression::escape(m_parameters.text));
         if (m_parameters.flags & Core::FindWholeWords)
             findString = QString::fromLatin1("\\b%1\\b").arg(findString);
-        QRegExp matcher(findString, (m_parameters.flags & Core::FindCaseSensitively
-                                     ? Qt::CaseSensitive : Qt::CaseInsensitive));
+        QRegularExpression matcher(findString, (m_parameters.flags & Core::FindCaseSensitively
+                                                ? QRegularExpression::NoPatternOption
+                                                : QRegularExpression::CaseInsensitiveOption));
+        matcher.optimize();
         while (it != m_snapshot.end()) {
             if (future.isPaused())
                 future.waitForResume();
@@ -296,7 +295,7 @@ public:
             if (m_fileNames.isEmpty() || m_fileNames.contains(it.value()->fileName())) {
                 QVector<Core::SearchResultItem> resultItems;
                 auto filter = [&](const IndexItem::Ptr &info) -> IndexItem::VisitorResult {
-                    if (matcher.indexIn(info->symbolName()) != -1) {
+                    if (matcher.match(info->symbolName()).hasMatch()) {
                         QString text = info->symbolName();
                         QString scope = info->symbolScope();
                         if (info->type() == IndexItem::Function) {
@@ -310,10 +309,7 @@ public:
                         Core::SearchResultItem item;
                         item.path = scope.split(QLatin1String("::"), QString::SkipEmptyParts);
                         item.text = text;
-                        item.textMarkPos = -1;
-                        item.textMarkLength = 0;
                         item.icon = info->icon();
-                        item.lineNumber = -1;
                         item.userData = qVariantFromValue(info);
                         resultItems << item;
                     }
@@ -345,20 +341,22 @@ BuiltinIndexingSupport::BuiltinIndexingSupport()
     m_synchronizer.setCancelOnWait(true);
 }
 
-BuiltinIndexingSupport::~BuiltinIndexingSupport()
-{}
+BuiltinIndexingSupport::~BuiltinIndexingSupport() = default;
 
-QFuture<void> BuiltinIndexingSupport::refreshSourceFiles(const QSet<QString> &sourceFiles,
-    CppModelManager::ProgressNotificationMode mode)
+QFuture<void> BuiltinIndexingSupport::refreshSourceFiles(
+        const QFutureInterface<void> &superFuture,
+        const QSet<QString> &sourceFiles,
+        CppModelManager::ProgressNotificationMode mode)
 {
     CppModelManager *mgr = CppModelManager::instance();
 
     ParseParams params;
+    params.indexerFileSizeLimitInMb = indexerFileSizeLimitInMb();
     params.headerPaths = mgr->headerPaths();
     params.workingCopy = mgr->workingCopy();
     params.sourceFiles = sourceFiles;
 
-    QFuture<void> result = QtConcurrent::run(&parse, params);
+    QFuture<void> result = Utils::runAsync(mgr->sharedThreadPool(), parse, superFuture, params);
 
     if (m_synchronizer.futures().size() > 10) {
         QList<QFuture<void> > futures = m_synchronizer.futures();
@@ -381,7 +379,8 @@ QFuture<void> BuiltinIndexingSupport::refreshSourceFiles(const QSet<QString> &so
     return result;
 }
 
-SymbolSearcher *BuiltinIndexingSupport::createSymbolSearcher(SymbolSearcher::Parameters parameters, QSet<QString> fileNames)
+SymbolSearcher *BuiltinIndexingSupport::createSymbolSearcher(
+        const SymbolSearcher::Parameters &parameters, const QSet<QString> &fileNames)
 {
     return new BuiltinSymbolSearcher(CppModelManager::instance()->snapshot(), parameters, fileNames);
 }
