@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,29 +9,25 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "cdbparsehelpers.h"
 
-#include "bytearrayinputstream.h"
+#include "stringinputstream.h"
 
+#include <debugger/breakhandler.h>
 #include <debugger/debuggerprotocol.h>
 #include <debugger/disassemblerlines.h>
 #include <debugger/shared/hostutils.h>
@@ -60,17 +56,17 @@ QString cdbSourcePathMapping(QString fileName,
                              const QList<QPair<QString, QString> > &sourcePathMapping,
                              SourcePathMode mode)
 {
-    typedef QPair<QString, QString> SourcePathMapping;
+    using SourcePathMapping = QPair<QString, QString>;
 
     if (fileName.isEmpty() || sourcePathMapping.isEmpty())
         return fileName;
-    foreach (const SourcePathMapping &m, sourcePathMapping) {
+    for (const SourcePathMapping &m : sourcePathMapping) {
         const QString &source = mode == DebuggerToSource ? m.first : m.second;
         const int sourceSize = source.size();
         // Map parts of the path and ensure a slash follows.
         if (fileName.size() > sourceSize && fileName.startsWith(source, Qt::CaseInsensitive)) {
             const QChar next = fileName.at(sourceSize);
-            if (next == QLatin1Char('\\') || next == QLatin1Char('/')) {
+            if (next == '\\' || next == '/') {
                 const QString &target = mode == DebuggerToSource ? m.second: m.first;
                 fileName.replace(0, sourceSize, target);
                 return fileName;
@@ -82,14 +78,14 @@ QString cdbSourcePathMapping(QString fileName,
 
 // Determine file name to be used for breakpoints. Convert to native and, unless short path
 // is set, perform reverse lookup in the source path mappings.
-static inline QString cdbBreakPointFileName(const BreakpointParameters &bp,
+static inline QString cdbBreakPointFileName(const BreakpointParameters &params,
                                             const QList<QPair<QString, QString> > &sourcePathMapping)
 {
-    if (bp.fileName.isEmpty())
-        return bp.fileName;
-    if (bp.pathUsage == BreakpointUseShortPath)
-        return Utils::FileName::fromString(bp.fileName).fileName();
-    return cdbSourcePathMapping(QDir::toNativeSeparators(bp.fileName), sourcePathMapping, SourceToDebugger);
+    if (params.fileName.isEmpty())
+        return params.fileName;
+    if (params.pathUsage == BreakpointUseShortPath)
+        return Utils::FileName::fromString(params.fileName).fileName();
+    return cdbSourcePathMapping(QDir::toNativeSeparators(params.fileName), sourcePathMapping, SourceToDebugger);
 }
 
 static BreakpointParameters fixWinMSVCBreakpoint(const BreakpointParameters &p)
@@ -109,102 +105,58 @@ static BreakpointParameters fixWinMSVCBreakpoint(const BreakpointParameters &p)
         break;
     case BreakpointAtExec: { // Emulate by breaking on CreateProcessW().
         BreakpointParameters rc(BreakpointByFunction);
-        rc.module = QLatin1String("kernel32");
-        rc.functionName = QLatin1String("CreateProcessW");
+        rc.module = "kernel32";
+        rc.functionName = "CreateProcessW";
         return rc;
     }
     case BreakpointAtThrow: {
         BreakpointParameters rc(BreakpointByFunction);
-        rc.functionName = QLatin1String("CxxThrowException"); // MSVC runtime. Potentially ambiguous.
+        rc.functionName = "CxxThrowException"; // MSVC runtime. Potentially ambiguous.
         return rc;
     }
     case BreakpointAtCatch: {
         BreakpointParameters rc(BreakpointByFunction);
-        rc.functionName = QLatin1String("__CxxCallCatchBlock"); // MSVC runtime. Potentially ambiguous.
+        rc.functionName = "__CxxCallCatchBlock"; // MSVC runtime. Potentially ambiguous.
         return rc;
     }
     case BreakpointAtMain: {
         BreakpointParameters rc(BreakpointByFunction);
-        rc.functionName = QLatin1String("main");
+        rc.functionName = "main";
         return rc;
     }
     } // switch
     return p;
 }
 
-int breakPointIdToCdbId(const BreakpointModelId &id)
+QString breakPointCdbId(const Breakpoint &bp)
 {
-    return cdbBreakPointStartId + id.majorPart() * cdbBreakPointIdMinorPart + id.minorPart();
+    static int bpId = 1;
+    if (!bp->responseId().isEmpty())
+        return bp->responseId();
+    return QString::number(cdbBreakPointStartId + (bpId++) * cdbBreakPointIdMinorPart);
 }
 
-template <class ModelId>
-inline ModelId cdbIdToBreakpointId(const int &id)
+QString cdbAddBreakpointCommand(const BreakpointParameters &bpIn,
+                                const QList<QPair<QString, QString> > &sourcePathMapping,
+                                const QString &responseId,
+                                bool oneshot)
 {
-    if (id >= cdbBreakPointStartId) {
-        int major = (id - cdbBreakPointStartId) / cdbBreakPointIdMinorPart;
-        int minor = id % cdbBreakPointIdMinorPart;
-        if (minor)
-            return ModelId(major, minor);
-        else
-            return ModelId(major);
-    }
-    return ModelId();
-}
+    const BreakpointParameters params = fixWinMSVCBreakpoint(bpIn);
+    QString rc;
+    StringInputStream str(rc);
 
-template <class ModelId>
-inline ModelId cdbIdToBreakpointId(const GdbMi &data)
-{
-    if (data.isValid()) { // Might not be valid if there is not id
-        bool ok;
-        const int id = data.data().toInt(&ok);
-        if (ok)
-            return cdbIdToBreakpointId<ModelId>(id);
-    }
-    return ModelId();
-}
-
-BreakpointModelId cdbIdToBreakpointModelId(const GdbMi &id)
-{
-    return cdbIdToBreakpointId<BreakpointModelId>(id);
-}
-
-BreakpointModelId cdbIdToBreakpointModelId(int id)
-{
-    return cdbIdToBreakpointId<BreakpointModelId>(id);
-}
-
-BreakpointResponseId cdbIdToBreakpointResponseId(const GdbMi &id)
-{
-    return cdbIdToBreakpointId<BreakpointResponseId>(id);
-}
-
-BreakpointResponseId cdbIdToBreakpointResponseId(int id)
-{
-    return cdbIdToBreakpointId<BreakpointResponseId>(id);
-}
-
-QByteArray cdbAddBreakpointCommand(const BreakpointParameters &bpIn,
-                                   const QList<QPair<QString, QString> > &sourcePathMapping,
-                                   BreakpointModelId id /* = BreakpointId() */,
-                                   bool oneshot)
-{
-    const BreakpointParameters bp = fixWinMSVCBreakpoint(bpIn);
-    QByteArray rc;
-    ByteArrayInputStream str(rc);
-
-    if (bp.threadSpec >= 0)
-        str << '~' << bp.threadSpec << ' ';
+    if (params.threadSpec >= 0)
+        str << '~' << params.threadSpec << ' ';
 
     // Currently use 'bu' so that the offset expression (including file name)
     // is kept when reporting back breakpoints (which is otherwise discarded
     // when resolving).
-    str << (bp.type == WatchpointAtAddress ? "ba" : "bu");
-    if (id.isValid())
-        str << breakPointIdToCdbId(id);
-    str << ' ';
+    str << (params.type == WatchpointAtAddress ? "ba" : "bu")
+        << responseId
+        << ' ';
     if (oneshot)
         str << "/1 ";
-    switch (bp.type) {
+    switch (params.type) {
     case BreakpointAtFork:
     case BreakpointAtExec:
     case WatchpointAtExpression:
@@ -216,44 +168,46 @@ QByteArray cdbAddBreakpointCommand(const BreakpointParameters &bpIn,
     case BreakpointAtJavaScriptThrow:
     case UnknownBreakpointType:
     case LastBreakpointType:
-        QTC_ASSERT(false, return QByteArray());
+        QTC_ASSERT(false, return QString());
         break;
     case BreakpointByAddress:
-        str << hex << hexPrefixOn << bp.address << hexPrefixOff << dec;
+        str << hex << hexPrefixOn << params.address << hexPrefixOff << dec;
         break;
     case BreakpointByFunction:
-        if (!bp.module.isEmpty())
-            str << bp.module << '!';
-        str << bp.functionName;
+        if (!params.module.isEmpty())
+            str << params.module << '!';
+        str << params.functionName;
         break;
     case BreakpointByFileAndLine:
         str << '`';
-        if (!bp.module.isEmpty())
-            str << bp.module << '!';
-        str << cdbBreakPointFileName(bp, sourcePathMapping) << ':' << bp.lineNumber << '`';
+        if (!params.module.isEmpty())
+            str << params.module << '!';
+        str << cdbBreakPointFileName(params, sourcePathMapping) << ':' << params.lineNumber << '`';
         break;
     case WatchpointAtAddress: { // Read/write, no space here
-        const unsigned size = bp.size ? bp.size : 1;
-        str << 'r' << size << ' ' << hex << hexPrefixOn << bp.address << hexPrefixOff << dec;
+        const unsigned size = params.size ? params.size : 1;
+        str << 'r' << size << ' ' << hex << hexPrefixOn << params.address << hexPrefixOff << dec;
     }
         break;
     }
-    if (bp.ignoreCount)
-        str << " 0n" << (bp.ignoreCount + 1);
+    if (params.ignoreCount)
+        str << " 0n" << (params.ignoreCount + 1);
     // Condition currently unsupported.
-    if (!bp.command.isEmpty())
-        str << " \"" << bp.command << '"';
+    if (!params.command.isEmpty())
+        str << " \"" << params.command << '"';
     return rc;
 }
 
-QByteArray cdbClearBreakpointCommand(const BreakpointModelId &id)
+QString cdbClearBreakpointCommand(const Breakpoint &bp)
 {
-    const int firstBreakPoint = breakPointIdToCdbId(id);
-    if (id.isMinor())
-        return "bc " + QByteArray::number(firstBreakPoint);
+// FIME: Check
+//    const int firstBreakPoint = breakPointCdbId(id);
+//    if (id.isMinor())
+//        return "bc " + QString::number(firstBreakPoint);
     // If this is a major break point we also want to delete all sub break points
+    const int firstBreakPoint = bp->responseId().toInt();
     const int lastBreakPoint = firstBreakPoint + cdbBreakPointIdMinorPart - 1;
-    return "bc " + QByteArray::number(firstBreakPoint) + '-' + QByteArray::number(lastBreakPoint);
+    return "bc " + QString::number(firstBreakPoint) + '-' + QString::number(lastBreakPoint);
 }
 
 // Helper to retrieve an int child from GDBMI
@@ -285,41 +239,38 @@ static inline bool gdbmiChildToBool(const GdbMi &parent, const char *childName,
 // Parse extension command listing breakpoints.
 // Note that not all fields are returned, since file, line, function are encoded
 // in the expression (that is in addition deleted on resolving for a bp-type breakpoint).
-void parseBreakPoint(const GdbMi &gdbmi, BreakpointResponse *r,
+void parseBreakPoint(const GdbMi &gdbmi, BreakpointParameters *r,
                      QString *expression /*  = 0 */)
 {
     gdbmiChildToBool(gdbmi, "enabled", &(r->enabled));
     gdbmiChildToBool(gdbmi, "deferred", &(r->pending));
-    r->id = BreakpointResponseId();
-    // Might not be valid if there is not id
-    r->id = cdbIdToBreakpointResponseId(gdbmi["id"]);
     const GdbMi moduleG = gdbmi["module"];
     if (moduleG.isValid())
-        r->module = QString::fromLocal8Bit(moduleG.data());
+        r->module = moduleG.data();
     const GdbMi sourceFileName = gdbmi["srcfile"];
     if (sourceFileName.isValid()) {
-        r->fileName = QString::fromLocal8Bit(sourceFileName.data());
+        r->fileName = sourceFileName.data();
         const GdbMi lineNumber = gdbmi["srcline"];
         if (lineNumber.isValid())
-            r->lineNumber = lineNumber.data().toULongLong(0, 0);
+            r->lineNumber = lineNumber.data().toULongLong(nullptr, 0);
     }
     if (expression) {
         const GdbMi expressionG = gdbmi["expression"];
         if (expressionG.isValid())
-            *expression = QString::fromLocal8Bit(expressionG.data());
+            *expression = expressionG.data();
     }
     const GdbMi addressG = gdbmi["address"];
     if (addressG.isValid())
-        r->address = addressG.data().toULongLong(0, 0);
+        r->address = addressG.data().toULongLong(nullptr, 0);
     if (gdbmiChildToInt(gdbmi, "passcount", &(r->ignoreCount)))
         r->ignoreCount--;
     gdbmiChildToInt(gdbmi, "thread", &(r->threadSpec));
 }
 
-QByteArray cdbWriteMemoryCommand(quint64 addr, const QByteArray &data)
+QString cdbWriteMemoryCommand(quint64 addr, const QByteArray &data)
 {
-    QByteArray cmd;
-    ByteArrayInputStream str(cmd);
+    QString cmd;
+    StringInputStream str(cmd);
     str.setIntegerBase(16);
     str << "f " << addr << " L" << data.size();
     const int count = data.size();
@@ -362,19 +313,7 @@ QString debugByteArray(const QByteArray &a)
     return rc;
 }
 
-QString StringFromBase64EncodedUtf16(const QByteArray &a)
-{
-    QByteArray utf16 = QByteArray::fromBase64(a);
-    utf16.append('\0');
-    utf16.append('\0');
-    return QString::fromUtf16(reinterpret_cast<const unsigned short *>(utf16.constData()));
-}
-
-WinException::WinException() :
-    exceptionCode(0), exceptionFlags(0), exceptionAddress(0),
-    info1(0),info2(0), firstChance(false), lineNumber(0)
-{
-}
+WinException::WinException() = default;
 
 void WinException::fromGdbMI(const GdbMi &gdbmi)
 {
@@ -387,7 +326,7 @@ void WinException::fromGdbMI(const GdbMi &gdbmi)
         info1 = ginfo1.data().toULongLong();
         const GdbMi ginfo2  = gdbmi["exceptionInformation1"];
         if (ginfo2.isValid())
-            info2 = ginfo1.data().toULongLong();
+            info2 = ginfo2.data().toULongLong();
     }
     const GdbMi gLineNumber = gdbmi["exceptionLine"];
     if (gLineNumber.isValid()) {
@@ -407,10 +346,10 @@ QString WinException::toString(bool includeLocation) const
         str << " (first chance)";
     if (includeLocation) {
         if (lineNumber) {
-            str << " at " << QLatin1String(file) << ':' << lineNumber;
+            str << " at " << file << ':' << lineNumber;
         } else {
             if (!function.isEmpty())
-                str << " in " << QLatin1String(function);
+                str << " in " << function;
         }
     }
     return rc;
@@ -463,25 +402,25 @@ bool parseCdbDisassemblerFunctionLine(const QString &l,
                                       QString *currentFunction, quint64 *functionOffset,
                                       QString *sourceFile)
 {
-    if (l.isEmpty() || !l.endsWith(QLatin1Char(':')) || l.at(0).isDigit() || l.at(0).isSpace())
+    if (l.isEmpty() || !l.endsWith(':') || l.at(0).isDigit() || l.at(0).isSpace())
         return false;
-    int functionEnd = l.indexOf(QLatin1Char(' '));
+    int functionEnd = l.indexOf(' ');
     if (functionEnd < 0)
         functionEnd = l.size() - 1; // Nothing at all, just ':'
-    const int offsetPos = l.indexOf(QLatin1String("+0x"));
+    const int offsetPos = l.indexOf("+0x");
     if (offsetPos > 0) {
         *currentFunction = l.left(offsetPos);
-        *functionOffset = l.mid(offsetPos + 3, functionEnd - offsetPos - 3).trimmed().toULongLong(0, 16);
+        *functionOffset = l.mid(offsetPos + 3, functionEnd - offsetPos - 3).trimmed().toULongLong(nullptr, 16);
     } else { // No offset, directly at beginning.
         *currentFunction = l.left(functionEnd);
         *functionOffset = 0;
     }
     sourceFile->clear();
     // Parse file and line.
-    const int filePos = l.indexOf(QLatin1Char('['), functionEnd);
+    const int filePos = l.indexOf('[', functionEnd);
     if (filePos == -1)
         return true; // No file
-    const int linePos = l.indexOf(QLatin1String(" @ "), filePos + 1);
+    const int linePos = l.indexOf(" @ ", filePos + 1);
     if (linePos == -1)
         return false;
     *sourceFile = l.mid(filePos + 1, linePos - filePos - 1).trimmed();
@@ -504,7 +443,7 @@ bool parseCdbDisassemblerLine(const QString &line, DisassemblerLine *dLine, uint
     *sourceLine = 0;
     if (line.size() < 6)
         return false;
-    const QChar blank = QLatin1Char(' ');
+    const QChar blank = ' ';
     int addressPos = 0;
     // Check for joined source and address in 6.11
     const bool hasV611SourceLine = line.at(5).isDigit();
@@ -538,7 +477,7 @@ bool parseCdbDisassemblerLine(const QString &line, DisassemblerLine *dLine, uint
     const int instructionPos = rawDataEnd + 1;
     bool ok;
     QString addressS = line.mid(addressPos, addressEnd - addressPos);
-    if (addressS.size() > 9 && addressS.at(8) == QLatin1Char('`'))
+    if (addressS.size() > 9 && addressS.at(8) == '`')
         addressS.remove(8, 1);
     dLine->address = addressS.toULongLong(&ok, 16);
     if (!ok)
@@ -548,7 +487,7 @@ bool parseCdbDisassemblerLine(const QString &line, DisassemblerLine *dLine, uint
     return true;
 }
 
-DisassemblerLines parseCdbDisassembler(const QList<QByteArray> &a)
+DisassemblerLines parseCdbDisassembler(const QString &a)
 {
     DisassemblerLines result;
     quint64 functionAddress = 0;
@@ -557,8 +496,7 @@ DisassemblerLines parseCdbDisassembler(const QList<QByteArray> &a)
     quint64 functionOffset = 0;
     QString sourceFile;
 
-    foreach (const QByteArray &lineBA, a) {
-        const QString line = QString::fromLatin1(lineBA);
+    foreach (const QString &line, a.split('\n')) {
         // New function. Append as comment line.
         if (parseCdbDisassemblerFunctionLine(line, &currentFunction, &functionOffset, &sourceFile)) {
             functionAddress = 0;
@@ -575,7 +513,7 @@ DisassemblerLines parseCdbDisassembler(const QList<QByteArray> &a)
                     result.appendSourceLine(sourceFile, sourceLine);
                 }
             } else {
-                qWarning("Unable to parse assembly line '%s'", lineBA.constData());
+                qWarning("Unable to parse assembly line '%s'", qPrintable(line));
                 disassemblyLine.fromString(line);
             }
             // Determine address of function from the first assembler line after a
