@@ -30,18 +30,21 @@
 #include "diffview.h"
 
 #include <coreplugin/coreconstants.h>
-#include <coreplugin/coreicons.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/minisplitter.h>
 
 #include <texteditor/displaysettings.h>
+#include <texteditor/fontsettings.h>
 #include <texteditor/marginsettings.h>
+#include <texteditor/syntaxhighlighter.h>
+#include <texteditor/textdocument.h>
 #include <texteditor/texteditor.h>
 #include <texteditor/texteditorsettings.h>
 
 #include <utils/algorithm.h>
 #include <utils/fileutils.h>
 #include <utils/qtcassert.h>
+#include <utils/utilsicons.h>
 
 #include <QAction>
 #include <QComboBox>
@@ -64,24 +67,8 @@ static const char ignoreWhitespaceKeyC[] = "IgnoreWhitespace";
 
 static const char diffViewKeyC[] = "DiffEditorType";
 
-static const char legacySettingsGroupC[] = "Git";
-static const char useDiffEditorKeyC[] = "UseDiffEditor";
-
 using namespace TextEditor;
 
-namespace {
-
-class Guard
-{
-public:
-    Guard(int *state) : m_state(state) { ++(*state); }
-    ~Guard() { --(*m_state); QTC_ASSERT(*m_state >= 0, return); }
-private:
-    int *m_state;
-};
-
-} // namespace
-
 namespace DiffEditor {
 namespace Internal {
 
@@ -89,25 +76,17 @@ class DescriptionEditorWidget : public TextEditorWidget
 {
     Q_OBJECT
 public:
-    DescriptionEditorWidget(QWidget *parent = 0);
-    virtual QSize sizeHint() const override;
+    DescriptionEditorWidget(QWidget *parent = nullptr);
+    ~DescriptionEditorWidget() override;
 
-signals:
-    void requestBranchList();
+    QSize sizeHint() const override;
 
 protected:
-    void mouseMoveEvent(QMouseEvent *e) override;
-    void mouseReleaseEvent(QMouseEvent *e) override;
-
     void setDisplaySettings(const DisplaySettings &ds) override;
     void setMarginSettings(const MarginSettings &ms) override;
 
-    bool findContentsUnderCursor(const QTextCursor &cursor);
-    void highlightCurrentContents();
-    void handleCurrentContents();
-
 private:
-    QTextCursor m_currentCursor;
+    Core::IContext *m_context;
 };
 
 DescriptionEditorWidget::DescriptionEditorWidget(QWidget *parent)
@@ -118,7 +97,6 @@ DescriptionEditorWidget::DescriptionEditorWidget(QWidget *parent)
     DisplaySettings settings = displaySettings();
     settings.m_textWrapping = false;
     settings.m_displayLineNumbers = false;
-    settings.m_highlightCurrentLine = false;
     settings.m_displayFoldingMarkers = false;
     settings.m_markTextChanges = false;
     settings.m_highlightBlocks = false;
@@ -128,6 +106,18 @@ DescriptionEditorWidget::DescriptionEditorWidget(QWidget *parent)
     setFrameStyle(QFrame::NoFrame);
 
     setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+
+    m_context = new Core::IContext(this);
+    m_context->setWidget(this);
+    m_context->setContext(Core::Context(Constants::C_DIFF_EDITOR_DESCRIPTION));
+    Core::ICore::addContextObject(m_context);
+
+    textDocument()->setSyntaxHighlighter(new SyntaxHighlighter);
+}
+
+DescriptionEditorWidget::~DescriptionEditorWidget()
+{
+    Core::ICore::removeContextObject(m_context);
 }
 
 QSize DescriptionEditorWidget::sizeHint() const
@@ -141,6 +131,8 @@ void DescriptionEditorWidget::setDisplaySettings(const DisplaySettings &ds)
 {
     DisplaySettings settings = displaySettings();
     settings.m_visualizeWhitespace = ds.m_visualizeWhitespace;
+    settings.m_scrollBarHighlights = ds.m_scrollBarHighlights;
+    settings.m_highlightCurrentLine = ds.m_highlightCurrentLine;
     TextEditorWidget::setDisplaySettings(settings);
 }
 
@@ -150,85 +142,9 @@ void DescriptionEditorWidget::setMarginSettings(const MarginSettings &ms)
     TextEditorWidget::setMarginSettings(MarginSettings());
 }
 
-void DescriptionEditorWidget::mouseMoveEvent(QMouseEvent *e)
-{
-    if (e->buttons()) {
-        TextEditorWidget::mouseMoveEvent(e);
-        return;
-    }
-
-    Qt::CursorShape cursorShape;
-
-    const QTextCursor cursor = cursorForPosition(e->pos());
-    if (findContentsUnderCursor(cursor)) {
-        highlightCurrentContents();
-        cursorShape = Qt::PointingHandCursor;
-    } else {
-        setExtraSelections(OtherSelection, QList<QTextEdit::ExtraSelection>());
-        cursorShape = Qt::IBeamCursor;
-    }
-
-    TextEditorWidget::mouseMoveEvent(e);
-    viewport()->setCursor(cursorShape);
-}
-
-void DescriptionEditorWidget::mouseReleaseEvent(QMouseEvent *e)
-{
-    if (e->button() == Qt::LeftButton && !(e->modifiers() & Qt::ShiftModifier)) {
-        const QTextCursor cursor = cursorForPosition(e->pos());
-        if (findContentsUnderCursor(cursor)) {
-            handleCurrentContents();
-            e->accept();
-            return;
-        }
-    }
-
-    TextEditorWidget::mouseReleaseEvent(e);
-}
-
-bool DescriptionEditorWidget::findContentsUnderCursor(const QTextCursor &cursor)
-{
-    m_currentCursor = cursor;
-    return cursor.block().text() == QLatin1String(Constants::EXPAND_BRANCHES);
-}
-
-void DescriptionEditorWidget::highlightCurrentContents()
-{
-    QTextEdit::ExtraSelection sel;
-    sel.cursor = m_currentCursor;
-    sel.cursor.select(QTextCursor::LineUnderCursor);
-    sel.format.setFontUnderline(true);
-    setExtraSelections(TextEditorWidget::OtherSelection,
-                       QList<QTextEdit::ExtraSelection>() << sel);
-}
-
-void DescriptionEditorWidget::handleCurrentContents()
-{
-    m_currentCursor.select(QTextCursor::LineUnderCursor);
-    m_currentCursor.removeSelectedText();
-    m_currentCursor.insertText(QLatin1String("Branches: Expanding..."));
-    emit requestBranchList();
-}
-
 ///////////////////////////////// DiffEditor //////////////////////////////////
 
 DiffEditor::DiffEditor()
-    : m_document(0)
-    , m_descriptionWidget(0)
-    , m_stackedWidget(0)
-    , m_toolBar(0)
-    , m_entriesComboBox(0)
-    , m_contextSpinBox(0)
-    , m_toggleSyncAction(0)
-    , m_whitespaceButtonAction(0)
-    , m_toggleDescriptionAction(0)
-    , m_reloadAction(0)
-    , m_viewSwitcherAction(0)
-    , m_currentViewIndex(-1)
-    , m_currentDiffFileIndex(-1)
-    , m_ignoreChanges(0)
-    , m_sync(false)
-    , m_showDescription(true)
 {
     // Editor:
     setDuplicateSupported(true);
@@ -243,8 +159,11 @@ DiffEditor::DiffEditor()
     m_stackedWidget = new QStackedWidget(splitter);
     splitter->addWidget(m_stackedWidget);
 
-    addView(new SideBySideView);
-    addView(new UnifiedView);
+    m_unifiedView = new UnifiedView;
+    m_sideBySideView = new SideBySideView;
+
+    addView(m_sideBySideView);
+    addView(m_unifiedView);
 
     setWidget(splitter);
 
@@ -264,17 +183,16 @@ DiffEditor::DiffEditor()
             this, &DiffEditor::setCurrentDiffFileIndex);
     m_toolBar->addWidget(m_entriesComboBox);
 
-    m_contextLabel = new QLabel(m_toolBar);
-
-    m_contextLabel->setText(tr("Context lines:"));
-    m_contextLabel->setContentsMargins(6, 0, 6, 0);
-    m_toolBar->addWidget(m_contextLabel);
+    QLabel *contextLabel = new QLabel(m_toolBar);
+    contextLabel->setText(tr("Context lines:"));
+    contextLabel->setContentsMargins(6, 0, 6, 0);
+    m_contextLabelAction = m_toolBar->addWidget(contextLabel);
 
     m_contextSpinBox = new QSpinBox(m_toolBar);
     m_contextSpinBox->setRange(1, 100);
     m_contextSpinBox->setFrame(false);
     m_contextSpinBox->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding); // Mac Qt5
-    m_toolBar->addWidget(m_contextSpinBox);
+    m_contextSpinBoxAction = m_toolBar->addWidget(m_contextSpinBox);
 
     m_whitespaceButtonAction = m_toolBar->addAction(tr("Ignore Whitespace"));
     m_whitespaceButtonAction->setCheckable(true);
@@ -282,10 +200,10 @@ DiffEditor::DiffEditor()
     m_toggleDescriptionAction = m_toolBar->addAction(Icons::TOP_BAR.icon(), QString());
     m_toggleDescriptionAction->setCheckable(true);
 
-    m_reloadAction = m_toolBar->addAction(Core::Icons::RELOAD.icon(), tr("Reload Diff"));
+    m_reloadAction = m_toolBar->addAction(Utils::Icons::RELOAD.icon(), tr("Reload Diff"));
     m_reloadAction->setToolTip(tr("Reload Diff"));
 
-    m_toggleSyncAction = m_toolBar->addAction(Core::Icons::LINK.icon(), QString());
+    m_toggleSyncAction = m_toolBar->addAction(Utils::Icons::LINK_TOOLBAR.icon(), QString());
     m_toggleSyncAction->setCheckable(true);
 
     m_viewSwitcherAction = m_toolBar->addAction(QIcon(), QString());
@@ -298,15 +216,13 @@ DiffEditor::DiffEditor()
     connect(m_viewSwitcherAction, &QAction::triggered, this, [this]() { showDiffView(nextView()); });
 }
 
-void DiffEditor::setDocument(QSharedPointer<DiffEditorDocument>(doc))
+void DiffEditor::setDocument(QSharedPointer<DiffEditorDocument> doc)
 {
     QTC_ASSERT(m_document.isNull(), return);
     QTC_ASSERT(doc, return);
 
-    m_document = QSharedPointer<DiffEditorDocument>(doc);
+    m_document = doc;
 
-    connect(m_descriptionWidget, &DescriptionEditorWidget::requestBranchList,
-            m_document.data(), &DiffEditorDocument::requestMoreInformation);
     connect(m_document.data(), &DiffEditorDocument::documentChanged,
             this, &DiffEditor::documentHasChanged);
     connect(m_document.data(), &DiffEditorDocument::descriptionChanged,
@@ -329,7 +245,7 @@ void DiffEditor::setDocument(QSharedPointer<DiffEditorDocument>(doc))
 
 DiffEditor::DiffEditor(DiffEditorDocument *doc) : DiffEditor()
 {
-    Guard guard(&m_ignoreChanges);
+    Utils::GuardLocker guard(m_ignoreChanges);
     setDocument(QSharedPointer<DiffEditorDocument>(doc));
     setupView(loadSettings());
 }
@@ -342,8 +258,8 @@ DiffEditor::~DiffEditor()
 
 Core::IEditor *DiffEditor::duplicate()
 {
-    DiffEditor *editor = new DiffEditor();
-    Guard guard(&editor->m_ignoreChanges);
+    auto editor = new DiffEditor();
+    Utils::GuardLocker guard(editor->m_ignoreChanges);
 
     editor->setDocument(m_document);
     editor->m_sync = m_sync;
@@ -358,30 +274,49 @@ Core::IEditor *DiffEditor::duplicate()
     return editor;
 }
 
-Core::IDocument *DiffEditor::document()
+Core::IDocument *DiffEditor::document() const
 {
     return m_document.data();
 }
 
 QWidget *DiffEditor::toolBar()
 {
-    QTC_ASSERT(m_toolBar, return 0);
+    QTC_ASSERT(m_toolBar, return nullptr);
     return m_toolBar;
 }
 
+TextEditorWidget *DiffEditor::descriptionWidget() const
+{
+    return m_descriptionWidget;
+}
+
+TextEditorWidget *DiffEditor::unifiedEditorWidget() const
+{
+    return m_unifiedView->textEditorWidget();
+}
+
+TextEditorWidget *DiffEditor::leftEditorWidget() const
+{
+    return m_sideBySideView->leftEditorWidget();
+}
+
+TextEditorWidget *DiffEditor::rightEditorWidget() const
+{
+    return m_sideBySideView->rightEditorWidget();
+}
+
 void DiffEditor::documentHasChanged()
 {
-    Guard guard(&m_ignoreChanges);
-    const QList<FileData> diffFileList = m_document->diffFiles();
+    Utils::GuardLocker guard(m_ignoreChanges);
+    const QList<FileData> &diffFileList = m_document->diffFiles();
 
     updateDescription();
     currentView()->setDiff(diffFileList, m_document->baseDirectory());
 
     m_entriesComboBox->clear();
-    const int count = diffFileList.count();
-    for (int i = 0; i < count; i++) {
-        const DiffFileInfo leftEntry = diffFileList.at(i).leftFileInfo;
-        const DiffFileInfo rightEntry = diffFileList.at(i).rightFileInfo;
+    for (const FileData &diffFile : diffFileList) {
+        const DiffFileInfo &leftEntry = diffFile.leftFileInfo;
+        const DiffFileInfo &rightEntry = diffFile.rightFileInfo;
         const QString leftShortFileName = Utils::FileName::fromString(leftEntry.fileName).fileName();
         const QString rightShortFileName = Utils::FileName::fromString(rightEntry.fileName).fileName();
         QString itemText;
@@ -430,11 +365,11 @@ void DiffEditor::documentHasChanged()
 
 void DiffEditor::toggleDescription()
 {
-    if (m_ignoreChanges > 0)
+    if (m_ignoreChanges.isLocked())
         return;
 
     m_showDescription = !m_showDescription;
-    saveSetting(QLatin1String(descriptionVisibleKeyC), m_showDescription);
+    saveSetting(descriptionVisibleKeyC, m_showDescription);
     updateDescription();
 }
 
@@ -446,7 +381,7 @@ void DiffEditor::updateDescription()
     m_descriptionWidget->setPlainText(description);
     m_descriptionWidget->setVisible(m_showDescription && !description.isEmpty());
 
-    Guard guard(&m_ignoreChanges);
+    Utils::GuardLocker guard(m_ignoreChanges);
     m_toggleDescriptionAction->setChecked(m_showDescription);
     m_toggleDescriptionAction->setToolTip(m_showDescription ? tr("Hide Change Description")
                                                       : tr("Show Change Description"));
@@ -458,11 +393,11 @@ void DiffEditor::updateDescription()
 void DiffEditor::contextLineCountHasChanged(int lines)
 {
     QTC_ASSERT(!m_document->isContextLineCountForced(), return);
-    if (m_ignoreChanges > 0 || lines == m_document->contextLineCount())
+    if (m_ignoreChanges.isLocked() || lines == m_document->contextLineCount())
         return;
 
     m_document->setContextLineCount(lines);
-    saveSetting(QLatin1String(contextLineCountKeyC), lines);
+    saveSetting(contextLineCountKeyC, lines);
 
     m_document->reload();
 }
@@ -471,10 +406,10 @@ void DiffEditor::ignoreWhitespaceHasChanged()
 {
     const bool ignore = m_whitespaceButtonAction->isChecked();
 
-    if (m_ignoreChanges > 0 || ignore == m_document->ignoreWhitespace())
+    if (m_ignoreChanges.isLocked() || ignore == m_document->ignoreWhitespace())
         return;
     m_document->setIgnoreWhitespace(ignore);
-    saveSetting(QLatin1String(ignoreWhitespaceKeyC), ignore);
+    saveSetting(ignoreWhitespaceKeyC, ignore);
 
     m_document->reload();
 }
@@ -494,7 +429,7 @@ void DiffEditor::prepareForReload()
     }
 
     {
-        Guard guard(&m_ignoreChanges);
+        Utils::GuardLocker guard(m_ignoreChanges);
         m_contextSpinBox->setValue(m_document->contextLineCount());
         m_whitespaceButtonAction->setChecked(m_document->ignoreWhitespace());
     }
@@ -510,11 +445,12 @@ void DiffEditor::reloadHasFinished(bool success)
 
     int index = -1;
     const QString startupFile = m_document->startupFile();
-    const QList<FileData> diffFileList = m_document->diffFiles();
+    const QList<FileData> &diffFileList = m_document->diffFiles();
     const int count = diffFileList.count();
     for (int i = 0; i < count; i++) {
-        const DiffFileInfo leftEntry = diffFileList.at(i).leftFileInfo;
-        const DiffFileInfo rightEntry = diffFileList.at(i).rightFileInfo;
+        const FileData &diffFile = diffFileList.at(i);
+        const DiffFileInfo &leftEntry = diffFile.leftFileInfo;
+        const DiffFileInfo &rightEntry = diffFile.rightFileInfo;
         if ((m_currentFileChunk.first.isEmpty()
              && m_currentFileChunk.second.isEmpty()
              && startupFile.endsWith(rightEntry.fileName))
@@ -539,12 +475,12 @@ void DiffEditor::updateEntryToolTip()
 
 void DiffEditor::setCurrentDiffFileIndex(int index)
 {
-    if (m_ignoreChanges > 0)
+    if (m_ignoreChanges.isLocked())
         return;
 
     QTC_ASSERT((index < 0) != (m_entriesComboBox->count() > 0), return);
 
-    Guard guard(&m_ignoreChanges);
+    Utils::GuardLocker guard(m_ignoreChanges);
     m_currentDiffFileIndex = index;
     currentView()->setCurrentDiffFileIndex(index);
 
@@ -558,8 +494,8 @@ void DiffEditor::documentStateChanged()
     const bool contextVisible = !m_document->isContextLineCountForced();
 
     m_whitespaceButtonAction->setVisible(canReload);
-    m_contextLabel->setVisible(canReload && contextVisible);
-    m_contextSpinBox->setVisible(canReload && contextVisible);
+    m_contextLabelAction->setVisible(canReload && contextVisible);
+    m_contextSpinBoxAction->setVisible(canReload && contextVisible);
     m_reloadAction->setVisible(canReload);
 }
 
@@ -575,41 +511,27 @@ void DiffEditor::updateDiffEditorSwitcher()
 
 void DiffEditor::toggleSync()
 {
-    if (m_ignoreChanges > 0)
+    if (m_ignoreChanges.isLocked())
         return;
 
     QTC_ASSERT(currentView(), return);
     m_sync = !m_sync;
-    saveSetting(QLatin1String(horizontalScrollBarSynchronizationKeyC), m_sync);
+    saveSetting(horizontalScrollBarSynchronizationKeyC, m_sync);
     currentView()->setSync(m_sync);
 }
 
 IDiffView *DiffEditor::loadSettings()
 {
-    QTC_ASSERT(currentView(), return 0);
+    QTC_ASSERT(currentView(), return nullptr);
     QSettings *s = Core::ICore::settings();
 
-    // TODO: Remove in 3.6: Read legacy settings first:
-    s->beginGroup(QLatin1String(legacySettingsGroupC));
-    const bool legacyExists = s->contains(QLatin1String(useDiffEditorKeyC));
-    const bool legacyEditor = s->value(
-                QLatin1String(useDiffEditorKeyC), true).toBool();
-    s->remove(QLatin1String(useDiffEditorKeyC));
-    s->endGroup();
-
-    // Save legacy settings to current settings:
-    if (legacyExists) {
-        saveSetting(QLatin1String(diffViewKeyC), legacyEditor ? m_views.at(0)->id().toSetting() :
-                                                                m_views.at(1)->id().toSetting());
-    }
-
     // Read current settings:
-    s->beginGroup(QLatin1String(settingsGroupC));
-    m_showDescription = s->value(QLatin1String(descriptionVisibleKeyC), true).toBool();
-    m_sync = s->value(QLatin1String(horizontalScrollBarSynchronizationKeyC), true).toBool();
-    m_document->setIgnoreWhitespace(s->value(QLatin1String(ignoreWhitespaceKeyC), false).toBool());
-    m_document->setContextLineCount(s->value(QLatin1String(contextLineCountKeyC), 3).toInt());
-    Core::Id id = Core::Id::fromSetting(s->value(QLatin1String(diffViewKeyC)));
+    s->beginGroup(settingsGroupC);
+    m_showDescription = s->value(descriptionVisibleKeyC, true).toBool();
+    m_sync = s->value(horizontalScrollBarSynchronizationKeyC, true).toBool();
+    m_document->setIgnoreWhitespace(s->value(ignoreWhitespaceKeyC, false).toBool());
+    m_document->setContextLineCount(s->value(contextLineCountKeyC, 3).toInt());
+    Core::Id id = Core::Id::fromSetting(s->value(diffViewKeyC));
     s->endGroup();
 
     IDiffView *view = Utils::findOr(m_views, m_views.at(0),
@@ -622,7 +544,7 @@ IDiffView *DiffEditor::loadSettings()
 void DiffEditor::saveSetting(const QString &key, const QVariant &value) const
 {
     QSettings *s = Core::ICore::settings();
-    s->beginGroup(QLatin1String(settingsGroupC));
+    s->beginGroup(settingsGroupC);
     s->setValue(key, value);
     s->endGroup();
 }
@@ -641,7 +563,7 @@ void DiffEditor::addView(IDiffView *view)
 IDiffView *DiffEditor::currentView() const
 {
     if (m_currentViewIndex < 0)
-        return 0;
+        return nullptr;
     return m_views.at(m_currentViewIndex);
 }
 
@@ -666,10 +588,10 @@ void DiffEditor::setupView(IDiffView *view)
     QTC_ASSERT(view, return);
     setCurrentView(view);
 
-    saveSetting(QLatin1String(diffViewKeyC), currentView()->id().toSetting());
+    saveSetting(diffViewKeyC, currentView()->id().toSetting());
 
     {
-        Guard guard(&m_ignoreChanges);
+        Utils::GuardLocker guard(m_ignoreChanges);
         m_toggleSyncAction->setVisible(currentView()->supportsSync());
         m_toggleSyncAction->setToolTip(currentView()->syncToolTip());
         m_toggleSyncAction->setText(currentView()->syncToolTip());
@@ -678,10 +600,6 @@ void DiffEditor::setupView(IDiffView *view)
 
     view->setDocument(m_document.data());
     view->setSync(m_sync);
-
-    view->beginOperation();
-    view->setDiff(m_document->diffFiles(), m_document->baseDirectory());
-    view->endOperation(true);
     view->setCurrentDiffFileIndex(m_currentDiffFileIndex);
 
     m_stackedWidget->setCurrentWidget(view->widget());
@@ -697,7 +615,7 @@ void DiffEditor::showDiffView(IDiffView *view)
         return;
 
     if (currentView()) // during initialization
-        currentView()->setDocument(0);
+        currentView()->setDocument(nullptr);
 
     QTC_ASSERT(view, return);
     setupView(view);
