@@ -27,9 +27,13 @@
 
 #include "projectexplorer_export.h"
 
+#include <utils/osspecificaspects.h>
+
 #include <QList>
 #include <QHash>
 
+#include <vector>
+
 namespace Utils { class FileName; }
 
 namespace ProjectExplorer {
@@ -49,6 +53,7 @@ public:
         PowerPCArchitecture,
         ShArchitecture,
         AvrArchitecture,
+        XtensaArchitecture,
         UnknownArchitecture
     };
 
@@ -71,14 +76,9 @@ public:
         OpenBsdFlavor,
 
         // Linux
-        GenericLinuxFlavor,
         AndroidLinuxFlavor,
 
-        // Darwin
-        GenericDarwinFlavor,
-
         // Unix
-        GenericUnixFlavor,
         SolarisUnixFlavor,
 
         // Windows
@@ -89,15 +89,18 @@ public:
         WindowsMsvc2013Flavor,
         WindowsMsvc2015Flavor,
         WindowsMsvc2017Flavor,
+        WindowsMsvc2019Flavor,
         WindowsMSysFlavor,
         WindowsCEFlavor,
 
         // Embedded
         VxWorksFlavor,
-        GenericQnxFlavor,
-        GenericBareMetalFlavor,
 
-        UnknownFlavor
+        // Generic:
+        RtosFlavor,
+        GenericFlavor,
+
+        UnknownFlavor // keep last in this enum!
     };
 
     enum BinaryFormat {
@@ -108,17 +111,14 @@ public:
         UnknownFormat
     };
 
-    Abi() :
-        m_architecture(UnknownArchitecture), m_os(UnknownOS),
-        m_osFlavor(UnknownFlavor), m_binaryFormat(UnknownFormat), m_wordWidth(0)
-    { }
-
-    Abi(const Architecture &a, const OS &o,
-        const OSFlavor &so, const BinaryFormat &f, unsigned char w);
-    Abi(const QString &abiString);
+    Abi(const Architecture &a = UnknownArchitecture, const OS &o = UnknownOS,
+        const OSFlavor &so = UnknownFlavor, const BinaryFormat &f = UnknownFormat,
+        unsigned char w = 0);
 
     static Abi abiFromTargetTriplet(const QString &machineTriple);
 
+    static Utils::OsType abiOsToOsType(const OS os);
+
     bool operator != (const Abi &other) const;
     bool operator == (const Abi &other) const;
     bool isCompatibleWith(const Abi &other) const;
@@ -140,12 +140,23 @@ public:
     static QString toString(const BinaryFormat &bf);
     static QString toString(int w);
 
+    static Architecture architectureFromString(const QStringRef &a);
+    static OS osFromString(const QStringRef &o);
+    static OSFlavor osFlavorFromString(const QStringRef &of, const OS os);
+    static BinaryFormat binaryFormatFromString(const QStringRef &bf);
+    static unsigned char wordWidthFromString(const QStringRef &w);
+
+    static OSFlavor registerOsFlavor(const std::vector<OS> &oses, const QString &flavorName);
     static QList<OSFlavor> flavorsForOs(const OS &o);
+    static QList<OSFlavor> allOsFlavors();
+    static bool osSupportsFlavor(const OS &os, const OSFlavor &flavor);
     static OSFlavor flavorForMsvcVersion(int version);
 
+    static Abi fromString(const QString &abiString);
     static Abi hostAbi();
     static QList<Abi> abisOfBinary(const Utils::FileName &path);
 
+
 private:
     Architecture m_architecture;
     OS m_os;
