@@ -29,15 +29,14 @@
 
 #include <coreplugin/icore.h>
 #include <coreplugin/messagemanager.h>
-#include <extensionsystem/pluginmanager.h>
-#include <projectexplorer/project.h>
+
 #include <projectexplorer/projectexplorerconstants.h>
-#include <ssh/sshhostkeydatabase.h>
+#include <utils/algorithm.h>
 #include <utils/fileutils.h>
 #include <utils/persistentsettings.h>
-#include <utils/qtcassert.h>
 #include <utils/portlist.h>
-#include <utils/algorithm.h>
+#include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 
 #include <QFileInfo>
 #include <QHash>
@@ -46,6 +45,7 @@
 #include <QVariantList>
 
 #include <limits>
+#include <memory>
 
 namespace ProjectExplorer {
 namespace Internal {
@@ -57,8 +57,7 @@ const char DefaultDevicesKey[] = "DefaultDevices";
 class DeviceManagerPrivate
 {
 public:
-    DeviceManagerPrivate() : writer(0)
-    { }
+    DeviceManagerPrivate() = default;
 
     int indexForId(Core::Id id) const
     {
@@ -72,17 +71,16 @@ public:
     static DeviceManager *clonedInstance;
     QList<IDevice::Ptr> devices;
     QHash<Core::Id, Core::Id> defaultDevices;
-    QSsh::SshHostKeyDatabasePtr hostKeyDatabase;
 
-    Utils::PersistentSettingsWriter *writer;
+    Utils::PersistentSettingsWriter *writer = nullptr;
 };
-DeviceManager *DeviceManagerPrivate::clonedInstance = 0;
+DeviceManager *DeviceManagerPrivate::clonedInstance = nullptr;
 
 } // namespace Internal
 
 using namespace Internal;
 
-DeviceManager *DeviceManager::m_instance = 0;
+DeviceManager *DeviceManager::m_instance = nullptr;
 
 DeviceManager *DeviceManager::instance()
 {
@@ -104,12 +102,12 @@ void DeviceManager::replaceInstance()
 void DeviceManager::removeClonedInstance()
 {
     delete DeviceManagerPrivate::clonedInstance;
-    DeviceManagerPrivate::clonedInstance = 0;
+    DeviceManagerPrivate::clonedInstance = nullptr;
 }
 
 DeviceManager *DeviceManager::cloneInstance()
 {
-    QTC_ASSERT(!DeviceManagerPrivate::clonedInstance, return 0);
+    QTC_ASSERT(!DeviceManagerPrivate::clonedInstance, return nullptr);
 
     DeviceManagerPrivate::clonedInstance = new DeviceManager(false);
     copy(instance(), DeviceManagerPrivate::clonedInstance, true);
@@ -134,7 +132,6 @@ void DeviceManager::save()
     QVariantMap data;
     data.insert(QLatin1String(DeviceManagerKey), toMap());
     d->writer->save(data, Core::ICore::mainWindow());
-    d->hostKeyDatabase->store(hostKeysFilePath());
 }
 
 void DeviceManager::load()
@@ -150,7 +147,7 @@ void DeviceManager::load()
     // read devices file from global settings path
     QHash<Core::Id, Core::Id> defaultDevices;
     QList<IDevice::Ptr> sdkDevices;
-    if (reader.load(systemSettingsFilePath(QLatin1String("/qtcreator/devices.xml"))))
+    if (reader.load(systemSettingsFilePath(QLatin1String("/devices.xml"))))
         sdkDevices = fromMap(reader.restoreValues().value(DeviceManagerKey).toMap(), &defaultDevices);
     // read devices file from user settings path
     QList<IDevice::Ptr> userDevices;
@@ -199,8 +196,9 @@ QList<IDevice::Ptr> DeviceManager::fromMap(const QVariantMap &map,
         const IDeviceFactory * const factory = restoreFactory(map);
         if (!factory)
             continue;
-        const IDevice::Ptr device = factory->restore(map);
+        const IDevice::Ptr device = factory->construct();
         QTC_ASSERT(device, continue);
+        device->fromMap(map);
         devices << device;
     }
     return devices;
@@ -210,7 +208,7 @@ QVariantMap DeviceManager::toMap() const
 {
     QVariantMap map;
     QVariantMap defaultDeviceMap;
-    typedef QHash<Core::Id, Core::Id> TypeIdHash;
+    using TypeIdHash = QHash<Core::Id, Core::Id>;
     for (TypeIdHash::ConstIterator it = d->defaultDevices.constBegin();
              it != d->defaultDevices.constEnd(); ++it) {
         defaultDeviceMap.insert(it.key().toString(), it.value().toSetting());
@@ -230,9 +228,8 @@ Utils::FileName DeviceManager::settingsFilePath(const QString &extension)
 
 Utils::FileName DeviceManager::systemSettingsFilePath(const QString &deviceFileRelativePath)
 {
-    return Utils::FileName::fromString(
-              QFileInfo(Core::ICore::settings(QSettings::SystemScope)->fileName()).absolutePath()
-              + deviceFileRelativePath);
+    return Utils::FileName::fromString(Core::ICore::installerResourcePath()
+                                       + deviceFileRelativePath);
 }
 
 void DeviceManager::addDevice(const IDevice::ConstPtr &_device)
@@ -245,7 +242,7 @@ void DeviceManager::addDevice(const IDevice::ConstPtr &_device)
             names << tmp->displayName();
     }
 
-    device->setDisplayName(Project::makeUnique(device->displayName(), names));
+    device->setDisplayName(Utils::makeUniquelyNumbered(device->displayName(), names));
 
     const int pos = d->indexForId(device->id());
 
@@ -315,11 +312,6 @@ bool DeviceManager::isLoaded() const
     return d->writer;
 }
 
-QSsh::SshHostKeyDatabasePtr DeviceManager::hostKeyDatabase() const
-{
-    return d->hostKeyDatabase;
-}
-
 void DeviceManager::setDefaultDevice(Core::Id id)
 {
     QTC_ASSERT(this != instance(), return);
@@ -338,9 +330,10 @@ void DeviceManager::setDefaultDevice(Core::Id id)
 
 const IDeviceFactory *DeviceManager::restoreFactory(const QVariantMap &map)
 {
-    IDeviceFactory *factory = ExtensionSystem::PluginManager::getObject<IDeviceFactory>(
-        [&map](IDeviceFactory *factory) {
-            return factory->canRestore(map);
+    const Core::Id deviceType = IDevice::typeFromMap(map);
+    IDeviceFactory *factory = Utils::findOrDefault(IDeviceFactory::allDeviceFactories(),
+        [&map, deviceType](IDeviceFactory *factory) {
+            return factory->canRestore(map) && factory->deviceType() == deviceType;
         });
 
     if (!factory)
@@ -350,18 +343,11 @@ const IDeviceFactory *DeviceManager::restoreFactory(const QVariantMap &map)
     return factory;
 }
 
-DeviceManager::DeviceManager(bool isInstance) : d(new DeviceManagerPrivate)
+DeviceManager::DeviceManager(bool isInstance) : d(std::make_unique<DeviceManagerPrivate>())
 {
     if (isInstance) {
         QTC_ASSERT(!m_instance, return);
         m_instance = this;
-        d->hostKeyDatabase = QSsh::SshHostKeyDatabasePtr::create();
-        const QString keyFilePath = hostKeysFilePath();
-        if (QFileInfo::exists(keyFilePath)) {
-            QString error;
-            if (!d->hostKeyDatabase->load(keyFilePath, &error))
-                Core::MessageManager::write(error);
-        }
         connect(Core::ICore::instance(), &Core::ICore::saveSettingsRequested, this, &DeviceManager::save);
     }
 }
@@ -371,8 +357,7 @@ DeviceManager::~DeviceManager()
     if (d->clonedInstance != this)
         delete d->writer;
     if (m_instance == this)
-        m_instance = 0;
-    delete d;
+        m_instance = nullptr;
 }
 
 IDevice::ConstPtr DeviceManager::deviceAt(int idx) const
@@ -406,11 +391,6 @@ IDevice::ConstPtr DeviceManager::defaultDevice(Core::Id deviceType) const
     return id.isValid() ? find(id) : IDevice::ConstPtr();
 }
 
-QString DeviceManager::hostKeysFilePath()
-{
-    return settingsFilePath(QLatin1String("/ssh-hostkeys")).toString();
-}
-
 } // namespace ProjectExplorer
 
 
@@ -426,21 +406,23 @@ class TestDevice : public IDevice
 {
 public:
     TestDevice()
-        : IDevice(testTypeId(), AutoDetected, Hardware, Core::Id::fromString(QUuid::createUuid().toString())) {}
+    {
+        setupId(AutoDetected, Core::Id::fromString(QUuid::createUuid().toString()));
+        setType(testTypeId());
+        setMachineType(Hardware);
+    }
 
     static Core::Id testTypeId() { return "TestType"; }
 private:
-    TestDevice(const TestDevice &other) : IDevice(other) {}
+    TestDevice(const TestDevice &other) = default;
     QString displayType() const override { return QLatin1String("blubb"); }
-    IDeviceWidget *createWidget() override { return 0; }
-    QList<Core::Id> actionIds() const override { return QList<Core::Id>(); }
-    QString displayNameForActionId(Core::Id) const override { return QString(); }
-    void executeAction(Core::Id, QWidget *) override { }
+    IDeviceWidget *createWidget() override { return nullptr; }
     Ptr clone() const override { return Ptr(new TestDevice(*this)); }
     DeviceProcessSignalOperation::Ptr signalOperation() const override
     {
         return DeviceProcessSignalOperation::Ptr();
     }
+    Utils::OsType osType() const override { return Utils::HostOsInfo::hostOs(); }
 };
 
 void ProjectExplorerPlugin::testDeviceManager()
