@@ -52,7 +52,7 @@ namespace ProjectExplorer {
 
 Q_GLOBAL_STATIC(QThreadPool, s_extraCompilerThreadPool);
 Q_GLOBAL_STATIC(QList<ExtraCompilerFactory *>, factories);
-
+Q_GLOBAL_STATIC(QVector<ExtraCompilerFactoryObserver *>, observers);
 class ExtraCompilerPrivate
 {
 public:
@@ -72,7 +72,7 @@ public:
 
 ExtraCompiler::ExtraCompiler(const Project *project, const Utils::FileName &source,
                              const Utils::FileNameList &targets, QObject *parent) :
-    QObject(parent), d(new ExtraCompilerPrivate)
+    QObject(parent), d(std::make_unique<ExtraCompilerPrivate>())
 {
     d->project = project;
     d->source = source;
@@ -80,9 +80,6 @@ ExtraCompiler::ExtraCompiler(const Project *project, const Utils::FileName &sour
         d->contents.insert(target, QByteArray());
     d->timer.setSingleShot(true);
 
-    connect(d->project, &Project::activeTargetChanged, this, &ExtraCompiler::onActiveTargetChanged);
-    onActiveTargetChanged();
-
     connect(&d->timer, &QTimer::timeout, this, [this](){
         if (d->dirty && d->lastEditor) {
             d->dirty = false;
@@ -132,10 +129,7 @@ ExtraCompiler::ExtraCompiler(const Project *project, const Utils::FileName &sour
     }
 }
 
-ExtraCompiler::~ExtraCompiler()
-{
-    delete d;
-}
+ExtraCompiler::~ExtraCompiler() = default;
 
 const Project *ExtraCompiler::project() const
 {
@@ -254,41 +248,6 @@ void ExtraCompiler::onEditorAboutToClose(Core::IEditor *editor)
     d->lastEditor = nullptr;
 }
 
-void ExtraCompiler::onActiveTargetChanged()
-{
-    disconnect(d->activeBuildConfigConnection);
-    if (Target *target = d->project->activeTarget()) {
-        d->activeBuildConfigConnection = connect(
-                target, &Target::activeBuildConfigurationChanged,
-                this, &ExtraCompiler::onActiveBuildConfigurationChanged);
-        onActiveBuildConfigurationChanged();
-    } else {
-        disconnect(d->activeEnvironmentConnection);
-        setDirty();
-    }
-}
-
-void ExtraCompiler::onActiveBuildConfigurationChanged()
-{
-    disconnect(d->activeEnvironmentConnection);
-    Target *target = d->project->activeTarget();
-    QTC_ASSERT(target, return);
-    if (BuildConfiguration *bc = target->activeBuildConfiguration()) {
-        d->activeEnvironmentConnection = connect(
-                    bc, &BuildConfiguration::environmentChanged,
-                    this, &ExtraCompiler::setDirty);
-    } else {
-        d->activeEnvironmentConnection = connect(KitManager::instance(), &KitManager::kitUpdated,
-                                                 this, [this](Kit *kit) {
-            Target *target = d->project->activeTarget();
-            QTC_ASSERT(target, return);
-            if (kit == target->kit())
-                setDirty();
-        });
-    }
-    setDirty();
-}
-
 Utils::Environment ExtraCompiler::buildEnvironment() const
 {
     if (Target *target = project()->activeTarget()) {
@@ -351,7 +310,8 @@ void ExtraCompiler::setContent(const Utils::FileName &file, const QByteArray &co
     }
 }
 
-ExtraCompilerFactory::ExtraCompilerFactory(QObject *parent) : QObject(parent)
+ExtraCompilerFactory::ExtraCompilerFactory(QObject *parent)
+    : QObject(parent)
 {
     factories->append(this);
 }
@@ -361,6 +321,14 @@ ExtraCompilerFactory::~ExtraCompilerFactory()
     factories->removeAll(this);
 }
 
+void ExtraCompilerFactory::annouceCreation(const Project *project,
+                                           const Utils::FileName &source,
+                                           const Utils::FileNameList &targets)
+{
+    for (ExtraCompilerFactoryObserver *observer : *observers)
+        observer->newExtraCompiler(project, source, targets);
+}
+
 QList<ExtraCompilerFactory *> ExtraCompilerFactory::extraCompilerFactories()
 {
     return *factories();
@@ -496,4 +464,14 @@ void ProcessExtraCompiler::cleanUp()
     setCompileTime(QDateTime::currentDateTime());
 }
 
+ExtraCompilerFactoryObserver::ExtraCompilerFactoryObserver()
+{
+    observers->push_back(this);
+}
+
+ExtraCompilerFactoryObserver::~ExtraCompilerFactoryObserver()
+{
+    observers->removeOne(this);
+}
+
 } // namespace ProjectExplorer
