@@ -223,7 +223,7 @@ QVariantMap DefaultPropertyProvider::autoGeneratedProperties(const ProjectExplor
     QVariantMap data = defaultData;
 
     const QString sysroot = ProjectExplorer::SysRootKitInformation::sysRoot(k).toUserOutput();
-    if (ProjectExplorer::SysRootKitInformation::hasSysRoot(k))
+    if (!sysroot.isEmpty())
         data.insert(QLatin1String(QBS_SYSROOT), sysroot);
 
     ProjectExplorer::ToolChain *tcC
@@ -296,7 +296,7 @@ QVariantMap DefaultPropertyProvider::autoGeneratedProperties(const ProjectExplor
     }
     data.insert(QLatin1String(CPP_TOOLCHAINPATH), mainFileInfo.absolutePath());
 
-    if (ProjectExplorer::GccToolChain *gcc = dynamic_cast<ProjectExplorer::GccToolChain *>(mainTc)) {
+    if (auto gcc = dynamic_cast<ProjectExplorer::GccToolChain *>(mainTc)) {
         QStringList compilerFlags = gcc->platformCodeGenFlags();
         filterCompilerLinkerFlags(targetAbi, compilerFlags);
         data.insert(QLatin1String(CPP_PLATFORMCOMMONCOMPILERFLAGS), compilerFlags);
