@@ -82,8 +82,8 @@ public:
     QbsGroupNode(const qbs::GroupData &grp, const QString &productPath);
 
     bool supportsAction(ProjectExplorer::ProjectAction action, const Node *node) const final;
-    bool addFiles(const QStringList &filePaths, QStringList *notAdded = 0) override;
-    bool removeFiles(const QStringList &filePaths, QStringList *notRemoved = 0) override;
+    bool addFiles(const QStringList &filePaths, QStringList *notAdded = nullptr) override;
+    bool removeFiles(const QStringList &filePaths, QStringList *notRemoved = nullptr) override;
     bool renameFile(const QString &filePath, const QString &newFilePath) override;
 
     qbs::GroupData qbsGroupData() const { return m_qbsGroupData; }
@@ -106,9 +106,12 @@ public:
 
     bool showInSimpleTree() const override;
     bool supportsAction(ProjectExplorer::ProjectAction action, const Node *node) const final;
-    bool addFiles(const QStringList &filePaths, QStringList *notAdded = 0) override;
-    bool removeFiles(const QStringList &filePaths, QStringList *notRemoved = 0) override;
+    bool addFiles(const QStringList &filePaths, QStringList *notAdded = nullptr) override;
+    bool removeFiles(const QStringList &filePaths, QStringList *notRemoved = nullptr) override;
     bool renameFile(const QString &filePath, const QString &newFilePath) override;
+    QStringList targetApplications() const override;
+
+    QString buildKey() const override;
 
     const qbs::ProductData qbsProductData() const { return m_qbsProductData; }
 
