@@ -29,7 +29,8 @@
 #include <projectexplorer/abstractprocessstep.h>
 
 #include <QStringList>
-#include <QFutureWatcher>
+
+#include <memory>
 
 namespace Utils { class FileName; }
 
@@ -44,7 +45,6 @@ namespace QtSupport { class BaseQtVersion; }
 
 namespace QmakeProjectManager {
 class QmakeBuildConfiguration;
-class QmakeProject;
 
 namespace Internal {
 
@@ -52,8 +52,6 @@ namespace Ui { class QMakeStep; }
 
 class QMakeStepFactory : public ProjectExplorer::BuildStepFactory
 {
-    Q_OBJECT
-
 public:
     QMakeStepFactory();
 };
@@ -77,6 +75,8 @@ public:
     QStringList toArguments() const;
 
     // Actual data
+    QString sysRoot;
+    QString targetTriple;
     TargetArchConfig archConfig = NoArch;
     OsType osType = NoOsType;
     bool linkQmlDebuggingQQ2 = false;
@@ -114,14 +114,20 @@ public:
     explicit QMakeStep(ProjectExplorer::BuildStepList *parent);
 
     QmakeBuildConfiguration *qmakeBuildConfiguration() const;
-    bool init(QList<const BuildStep *> &earlierSteps) override;
-    void run(QFutureInterface<bool> &) override;
+    bool init() override;
+    void doRun() override;
     ProjectExplorer::BuildStepConfigWidget *createConfigWidget() override;
-    bool immutable() const override;
     void setForced(bool b);
 
+    enum class ArgumentFlag {
+        OmitProjectPath = 0x01,
+        Expand = 0x02
+    };
+    Q_DECLARE_FLAGS(ArgumentFlags, ArgumentFlag);
+
     // the complete argument line
-    QString allArguments(const QtSupport::BaseQtVersion *v, bool shorted = false) const;
+    QString allArguments(const QtSupport::BaseQtVersion *v,
+                         ArgumentFlags flags = ArgumentFlags()) const;
     QMakeStepConfig deducedArguments() const;
     // arguments passed to the pro file parser
     QStringList parserArguments();
@@ -140,7 +146,7 @@ public:
     void setSeparateDebugInfo(bool enable);
 
     QString makeCommand() const;
-    QString makeArguments() const;
+    QString makeArguments(const QString &makefile) const;
     QString effectiveQMakeCall() const;
 
     QVariantMap toMap() const override;
@@ -158,6 +164,9 @@ protected:
     bool processSucceeded(int exitCode, QProcess::ExitStatus status) override;
 
 private:
+    void doCancel() override;
+    void finish(bool success) override;
+
     void startOneCommand(const QString &command, const QString &args);
     void runNextCommand();
 
@@ -169,13 +178,9 @@ private:
     // Extra arguments for qmake.
     QStringList m_extraArgs;
 
-    QFutureInterface<bool> m_inputFuture;
-    QFutureWatcher<bool> m_inputWatcher;
-    std::unique_ptr<QFutureInterface<bool>> m_commandFuture;
-    QFutureWatcher<bool> m_commandWatcher;
-
     // last values
     enum class State { IDLE = 0, RUN_QMAKE, RUN_MAKE_QMAKE_ALL, POST_PROCESS };
+    bool m_wasSuccess = true;
     State m_nextState = State::IDLE;
     bool m_forced = false;
     bool m_needToRunQMake = false; // set in init(), read in run()
@@ -193,10 +198,8 @@ class QMakeStepConfigWidget : public ProjectExplorer::BuildStepConfigWidget
     Q_OBJECT
 public:
     QMakeStepConfigWidget(QMakeStep *step);
-    ~QMakeStepConfigWidget();
-    QString summaryText() const;
-    QString additionalSummaryText() const;
-    QString displayName() const;
+    ~QMakeStepConfigWidget() override;
+
 private:
     // slots for handling buildconfiguration/step signals
     void qtVersionChanged();
@@ -221,13 +224,11 @@ private:
     void updateQtQuickCompilerOption();
     void updateEffectiveQMakeCall();
 
-    void setSummaryText(const QString &);
-
     Internal::Ui::QMakeStep *m_ui = nullptr;
     QMakeStep *m_step = nullptr;
-    QString m_summaryText;
-    QString m_additionalSummaryText;
     bool m_ignoreChange = false;
 };
 
 } // namespace QmakeProjectManager
+
+Q_DECLARE_OPERATORS_FOR_FLAGS(QmakeProjectManager::QMakeStep::ArgumentFlags);
