@@ -42,7 +42,7 @@
 #include <QPen>
 #include <qdebug.h>
 
-static Q_LOGGING_CATEGORY(itemlibraryPopulate, "itemlibrary.populate")
+static Q_LOGGING_CATEGORY(itemlibraryPopulate, "qtc.itemlibrary.populate", QtWarningMsg)
 
 static bool inline registerItemLibrarySortedModel() {
     qmlRegisterType<QmlDesigner::ItemLibrarySectionModel>();
@@ -88,11 +88,11 @@ QVariant ItemLibraryModel::data(const QModelIndex &index, int role) const
     if (m_roleNames.contains(role)) {
         QVariant value = m_sections.at(index.row())->property(m_roleNames.value(role));
 
-        ItemLibrarySectionModel* model = qobject_cast<ItemLibrarySectionModel *>(value.value<QObject*>());
+        auto model = qobject_cast<ItemLibrarySectionModel *>(value.value<QObject*>());
         if (model)
             return QVariant::fromValue(model);
 
-        ItemLibraryModel* model2 = qobject_cast<ItemLibraryModel *>(value.value<QObject*>());
+        auto model2 = qobject_cast<ItemLibraryModel *>(value.value<QObject*>());
         if (model2)
             return QVariant::fromValue(model2);
 
@@ -126,7 +126,7 @@ void ItemLibraryModel::setSearchText(const QString &searchText)
         bool changed = false;
         updateVisibility(&changed);
         if (changed)
-            dataChanged(QModelIndex(), QModelIndex());
+            emit dataChanged(QModelIndex(), QModelIndex());
     }
 }
 
@@ -140,6 +140,14 @@ Import entryToImport(const ItemLibraryEntry &entry)
 
 }
 
+bool sectionExapanded(const QString &sectionName)
+{
+    if (collapsedStateHash.contains(sectionName))
+        return collapsedStateHash.value(sectionName);
+
+    return true;
+}
+
 void ItemLibraryModel::update(ItemLibraryInfo *itemLibraryInfo, Model *model)
 {
     if (!model)
@@ -184,12 +192,13 @@ void ItemLibraryModel::update(ItemLibraryInfo *itemLibraryInfo, Model *model)
             qCInfo(itemlibraryPopulate) << "Adding:" << entry.typeName() << "to:" << entry.category();
             ItemLibrarySection *sectionModel = sectionByName(itemSectionName);
 
-            if (sectionModel == 0) {
+            if (sectionModel == nullptr) {
                 sectionModel = new ItemLibrarySection(itemSectionName, this);
                 m_sections.append(sectionModel);
+                sectionModel->setSectionExpanded(sectionExapanded(itemSectionName));
             }
 
-            ItemLibraryItem *item = new ItemLibraryItem(sectionModel);
+            auto item = new ItemLibraryItem(sectionModel);
             item->setItemLibraryEntry(entry);
             sectionModel->addSectionEntry(item);
         }
@@ -203,7 +212,7 @@ void ItemLibraryModel::update(ItemLibraryInfo *itemLibraryInfo, Model *model)
 
 QMimeData *ItemLibraryModel::getMimeData(const ItemLibraryEntry &itemLibraryEntry)
 {
-    QMimeData *mimeData = new QMimeData();
+    auto mimeData = new QMimeData();
 
     QByteArray data;
     QDataStream stream(&data, QIODevice::WriteOnly);
@@ -239,7 +248,7 @@ ItemLibrarySection *ItemLibraryModel::sectionByName(const QString &sectionName)
             return itemLibrarySection;
     }
 
-    return 0;
+    return nullptr;
 }
 
 void ItemLibraryModel::updateVisibility(bool *changed)
