@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,57 +9,65 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "viewmanager.h"
 
+#ifndef QMLDESIGNER_TEST
+
+#include <componentaction.h>
+#include <designmodewidget.h>
+#include <crumblebar.h>
+#include <abstractview.h>
 #include <rewriterview.h>
 #include <nodeinstanceview.h>
 #include <itemlibraryview.h>
 #include <navigatorview.h>
 #include <stateseditorview.h>
 #include <formeditorview.h>
+#include <texteditorview.h>
 #include <propertyeditorview.h>
 #include <componentview.h>
 #include <debugview.h>
 #include <importmanagerview.h>
 #include <designeractionmanagerview.h>
+#include <qmldesignerplugin.h>
 
-#include "componentaction.h"
-#include "designmodewidget.h"
-#include "crumblebar.h"
-
-#include <qmldesigner/qmldesignerplugin.h>
 #include <utils/algorithm.h>
 
+#include <QLoggingCategory>
+#include <QTabWidget>
 
 namespace QmlDesigner {
 
+static Q_LOGGING_CATEGORY(viewBenchmark, "qtc.viewmanager.attach", QtWarningMsg)
+
 class ViewManagerData
 {
 public:
     QmlModelState savedState;
     Internal::DebugView debugView;
+    DesignerActionManagerView designerActionManagerView;
+    NodeInstanceView nodeInstanceView;
     ComponentView componentView;
     FormEditorView formEditorView;
+    TextEditorView textEditorView;
     ItemLibraryView itemLibraryView;
     NavigatorView navigatorView;
     PropertyEditorView propertyEditorView;
     StatesEditorView statesEditorView;
-    NodeInstanceView nodeInstanceView;
-    DesignerActionManagerView designerActionManagerView;
 
     QList<QPointer<AbstractView> > additionalViews;
 };
@@ -72,6 +80,11 @@ static CrumbleBar *crumbleBar() {
 ViewManager::ViewManager()
     : d(new ViewManagerData)
 {
+    d->formEditorView.setGotoErrorCallback([this](int line, int column) {
+        d->textEditorView.gotoCursorPosition(line, column);
+        if (Internal::DesignModeWidget *designModeWidget = QmlDesignerPlugin::instance()->mainWidget())
+            designModeWidget->showInternalTextEditor();
+    });
 }
 
 ViewManager::~ViewManager()
@@ -89,23 +102,48 @@ DesignDocument *ViewManager::currentDesignDocument() const
 
 void ViewManager::attachNodeInstanceView()
 {
+
+    QTime time;
+    if (viewBenchmark().isInfoEnabled())
+        time.start();
+
+    qCInfo(viewBenchmark) << Q_FUNC_INFO;
+
     setNodeInstanceViewKit(currentDesignDocument()->currentKit());
     currentModel()->setNodeInstanceView(&d->nodeInstanceView);
+
+     qCInfo(viewBenchmark) << "NodeInstanceView:" << time.elapsed();
 }
 
 void ViewManager::attachRewriterView()
 {
-    if (currentDesignDocument()->rewriterView()) {
-        currentModel()->setRewriterView(currentDesignDocument()->rewriterView());
-        currentDesignDocument()->rewriterView()->reactivateTextMofifierChangeSignals();
+    QTime time;
+    if (viewBenchmark().isInfoEnabled())
+        time.start();
+
+    qCInfo(viewBenchmark) << Q_FUNC_INFO;
+
+    if (RewriterView *view = currentDesignDocument()->rewriterView()) {
+        view->setWidgetStatusCallback([this](bool enable) {
+            if (enable)
+                enableWidgets();
+            else
+                disableWidgets();
+        });
+
+        currentModel()->setRewriterView(view);
+        view->reactivateTextMofifierChangeSignals();
+        view->restoreAuxiliaryData();
     }
+
+    qCInfo(viewBenchmark) << "RewriterView:" << time.elapsed();
 }
 
 void ViewManager::detachRewriterView()
 {
-    if (currentDesignDocument()->rewriterView()) {
-        currentDesignDocument()->rewriterView()->deactivateTextMofifierChangeSignals();
-        currentModel()->setRewriterView(0);
+    if (RewriterView *view = currentDesignDocument()->rewriterView()) {
+        view->deactivateTextMofifierChangeSignals();
+        currentModel()->setRewriterView(nullptr);
     }
 }
 
@@ -123,6 +161,20 @@ void ViewManager::switchStateEditorViewToSavedState()
         d->statesEditorView.setCurrentState(d->savedState);
 }
 
+QList<QPointer<AbstractView> > ViewManager::views() const
+{
+    auto list = d->additionalViews;
+    list.append({
+                    &d->formEditorView,
+                    &d->textEditorView,
+                    &d->itemLibraryView,
+                    &d->navigatorView,
+                    &d->propertyEditorView,
+                    &d->statesEditorView
+                });
+    return list;
+}
+
 void ViewManager::resetPropertyEditorView()
 {
     d->propertyEditorView.resetView();
@@ -144,6 +196,7 @@ void ViewManager::detachViewsExceptRewriterAndComponetView()
     detachAdditionalViews();
     currentModel()->detachView(&d->designerActionManagerView);
     currentModel()->detachView(&d->formEditorView);
+    currentModel()->detachView(&d->textEditorView);
     currentModel()->detachView(&d->navigatorView);
     currentModel()->detachView(&d->itemLibraryView);
     currentModel()->detachView(&d->statesEditorView);
@@ -152,7 +205,7 @@ void ViewManager::detachViewsExceptRewriterAndComponetView()
     if (d->debugView.isAttached())
         currentModel()->detachView(&d->debugView);
 
-    currentModel()->setNodeInstanceView(0);
+    currentModel()->setNodeInstanceView(nullptr);
 }
 
 void ViewManager::attachItemLibraryView()
@@ -176,31 +229,87 @@ void ViewManager::detachAdditionalViews()
 void ViewManager::attachComponentView()
 {
     documentModel()->attachView(&d->componentView);
-    QObject::connect(d->componentView.action(), SIGNAL(currentComponentChanged(ModelNode)), currentDesignDocument(), SLOT(changeToSubComponent(ModelNode)));
-    QObject::connect(d->componentView.action(), SIGNAL(changedToMaster()), currentDesignDocument(), SLOT(changeToMaster()));
+    QObject::connect(d->componentView.action(), &ComponentAction::currentComponentChanged,
+                     currentDesignDocument(), &DesignDocument::changeToSubComponent);
+    QObject::connect(d->componentView.action(), &ComponentAction::changedToMaster,
+                     currentDesignDocument(), &DesignDocument::changeToMaster);
 }
 
 void ViewManager::detachComponentView()
 {
-    QObject::disconnect(d->componentView.action(), SIGNAL(currentComponentChanged(ModelNode)), currentDesignDocument(), SLOT(changeToSubComponent(ModelNode)));
-    QObject::disconnect(d->componentView.action(), SIGNAL(changedToMaster()), currentDesignDocument(), SLOT(changeToMaster()));
+    QObject::disconnect(d->componentView.action(), &ComponentAction::currentComponentChanged,
+                        currentDesignDocument(), &DesignDocument::changeToSubComponent);
+    QObject::disconnect(d->componentView.action(), &ComponentAction::changedToMaster,
+                        currentDesignDocument(), &DesignDocument::changeToMaster);
 
     documentModel()->detachView(&d->componentView);
 }
 
 void ViewManager::attachViewsExceptRewriterAndComponetView()
 {
-    if (QmlDesignerPlugin::instance()->settings().enableDebugView)
+    if (QmlDesignerPlugin::instance()->settings().value(
+            DesignerSettingsKey::ENABLE_DEBUGVIEW).toBool())
         currentModel()->attachView(&d->debugView);
 
     attachNodeInstanceView();
-    currentModel()->attachView(&d->formEditorView);
-    currentModel()->attachView(&d->navigatorView);
-    attachItemLibraryView();
-    currentModel()->attachView(&d->statesEditorView);
-    currentModel()->attachView(&d->propertyEditorView);
+
+    QTime time;
+    if (viewBenchmark().isInfoEnabled())
+        time.start();
+
+    qCInfo(viewBenchmark) << Q_FUNC_INFO;
+
     currentModel()->attachView(&d->designerActionManagerView);
+
+    int last = time.elapsed();
+    qCInfo(viewBenchmark) << "ActionManagerView:" << last << time.elapsed();
+
+    currentModel()->attachView(&d->formEditorView);
+
+    int currentTime = time.elapsed();
+    qCInfo(viewBenchmark) << "FormEditorView:" << currentTime - last;
+    last = currentTime;
+
+    currentModel()->attachView(&d->textEditorView);
+
+    currentTime = time.elapsed();
+    qCInfo(viewBenchmark) << "TextEditorView:" << currentTime - last;
+    last = currentTime;
+
+    currentModel()->attachView(&d->navigatorView);
+
+    currentTime = time.elapsed();
+    qCInfo(viewBenchmark) << "NavigatorView:" << currentTime - last;
+    last = currentTime;
+
+    attachItemLibraryView();
+
+    currentTime = time.elapsed();
+    qCInfo(viewBenchmark) << "ItemLibraryView:" << currentTime - last;
+    last = currentTime;
+
+    currentModel()->attachView(&d->statesEditorView);
+
+    currentTime = time.elapsed();
+    qCInfo(viewBenchmark) << "StatesEditorView:" << currentTime - last;
+    last = currentTime;
+
+    currentModel()->attachView(&d->propertyEditorView);
+
+    currentTime = time.elapsed();
+    qCInfo(viewBenchmark) << "PropertyEditorView:" << currentTime - last;
+    last = currentTime;
+
     attachAdditionalViews();
+
+    currentTime = time.elapsed();
+    qCInfo(viewBenchmark) << "AdditionalViews:" << currentTime - last;
+    last = currentTime;
+
+    currentTime = time.elapsed();
+    qCInfo(viewBenchmark) << "All:" << time.elapsed();
+    last = currentTime;
+
     switchStateEditorViewToSavedState();
 }
 
@@ -224,11 +333,17 @@ void ViewManager::setNodeInstanceViewKit(ProjectExplorer::Kit *kit)
     d->nodeInstanceView.setKit(kit);
 }
 
-QList<WidgetInfo> ViewManager::widgetInfos()
+void QmlDesigner::ViewManager::setNodeInstanceViewProject(ProjectExplorer::Project *project)
+{
+    d->nodeInstanceView.setProject(project);
+}
+
+QList<WidgetInfo> ViewManager::widgetInfos() const
 {
     QList<WidgetInfo> widgetInfoList;
 
     widgetInfoList.append(d->formEditorView.widgetInfo());
+    widgetInfoList.append(d->textEditorView.widgetInfo());
     widgetInfoList.append(d->itemLibraryView.widgetInfo());
     widgetInfoList.append(d->navigatorView.widgetInfo());
     widgetInfoList.append(d->propertyEditorView.widgetInfo());
@@ -248,19 +363,28 @@ QList<WidgetInfo> ViewManager::widgetInfos()
     return widgetInfoList;
 }
 
+QWidget *ViewManager::widget(const QString &uniqueId) const
+{
+    foreach (const WidgetInfo &widgetInfo, widgetInfos()) {
+        if (widgetInfo.uniqueId == uniqueId)
+            return widgetInfo.widget;
+    }
+    return nullptr;
+}
+
 void ViewManager::disableWidgets()
 {
-    foreach (const WidgetInfo &widgetInfo, widgetInfos())
-        widgetInfo.widget->setEnabled(false);
+    for (const auto &view : views())
+        view->disableWidget();
 }
 
 void ViewManager::enableWidgets()
 {
-    foreach (const WidgetInfo &widgetInfo, widgetInfos())
-        widgetInfo.widget->setEnabled(true);
+    for (const auto &view : views())
+        view->enableWidget();
 }
 
-void ViewManager::pushFileOnCrumbleBar(const QString &fileName)
+void ViewManager::pushFileOnCrumbleBar(const Utils::FileName &fileName)
 {
     crumbleBar()->pushFile(fileName);
 }
@@ -295,6 +419,16 @@ const DesignerActionManager &ViewManager::designerActionManager() const
     return d->designerActionManagerView.designerActionManager();
 }
 
+void ViewManager::toggleStatesViewExpanded()
+{
+    d->statesEditorView.toggleStatesViewExpanded();
+}
+
+void ViewManager::qmlJSEditorContextHelp(const Core::IContext::HelpCallback &callback) const
+{
+    d->textEditorView.qmlJSEditorContextHelp(callback);
+}
+
 Model *ViewManager::currentModel() const
 {
     return currentDesignDocument()->currentModel();
@@ -305,4 +439,21 @@ Model *ViewManager::documentModel() const
     return currentDesignDocument()->documentModel();
 }
 
+void ViewManager::exportAsImage()
+{
+    d->formEditorView.exportAsImage();
+}
+
+void ViewManager::reformatFileUsingTextEditorView()
+{
+    d->textEditorView.reformatFile();
+}
+
+bool ViewManager::usesRewriterView(RewriterView *rewriterView)
+{
+    return currentDesignDocument()->rewriterView() == rewriterView;
+}
+
 } // namespace QmlDesigner
+
+#endif //QMLDESIGNER_TEST
