@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -34,8 +29,10 @@
 #include "qmljseditordocument.h"
 #include "qmlexpressionundercursor.h"
 
+#include <coreplugin/icore.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/helpitem.h>
 #include <coreplugin/helpmanager.h>
 #include <utils/qtcassert.h>
 #include <extensionsystem/pluginmanager.h>
@@ -48,12 +45,14 @@
 #include <qmljs/qmljsutils.h>
 #include <qmljs/qmljsqrcparser.h>
 #include <texteditor/texteditor.h>
-#include <texteditor/helpitem.h>
+#include <utils/executeondestruction.h>
 #include <utils/tooltip/tooltip.h>
 
 #include <QDir>
 #include <QList>
 #include <QStringRef>
+#include <QRegularExpression>
+#include <QRegularExpressionMatch>
 
 using namespace Core;
 using namespace QmlJS;
@@ -73,11 +72,10 @@ namespace {
 
     AST::UiObjectInitializer *nodeInitializer(AST::Node *node)
     {
-        AST::UiObjectInitializer *initializer = 0;
-        if (const AST::UiObjectBinding *binding = AST::cast<const AST::UiObjectBinding *>(node))
+        AST::UiObjectInitializer *initializer = nullptr;
+        if (auto binding = AST::cast<const AST::UiObjectBinding*>(node))
             initializer = binding->initializer;
-         else if (const AST::UiObjectDefinition *definition =
-                  AST::cast<const AST::UiObjectDefinition *>(node))
+         else if (auto definition = AST::cast<const AST::UiObjectDefinition*>(node))
             initializer = definition->initializer;
         return initializer;
     }
@@ -93,7 +91,7 @@ namespace {
     }
 }
 
-QmlJSHoverHandler::QmlJSHoverHandler() : m_modelManager(0)
+QmlJSHoverHandler::QmlJSHoverHandler()
 {
     m_modelManager = ModelManagerInterface::instance();
 }
@@ -148,42 +146,63 @@ bool QmlJSHoverHandler::setQmlTypeHelp(const ScopeChain &scopeChain, const Docum
                                        const ObjectValue *value, const QStringList &qName)
 {
     QString moduleName = getModuleName(scopeChain, qmlDocument, value);
-    QString helpId;
-    do {
-        QStringList helpIdPieces(qName);
-        helpIdPieces.prepend(moduleName);
-        helpIdPieces.prepend(QLatin1String("QML"));
-        helpId = helpIdPieces.join(QLatin1Char('.'));
-        if (!HelpManager::linksForIdentifier(helpId).isEmpty())
-            break;
-        if (helpIdPieces.size() > 3) {
-            QString lm = helpIdPieces.value(2);
-            helpIdPieces.removeAt(2);
-            helpId = helpIdPieces.join(QLatin1Char('.'));
-            if (!HelpManager::linksForIdentifier(helpId).isEmpty())
-                break;
-            helpIdPieces.replace(1, lm);
-            if (!HelpManager::linksForIdentifier(helpId).isEmpty())
-                break;
+
+    QStringList helpIdCandidates;
+
+    QStringList helpIdPieces(qName);
+    helpIdPieces.prepend(moduleName);
+    helpIdPieces.prepend("QML");
+    helpIdCandidates += helpIdPieces.join('.');
+
+    if (helpIdPieces.size() > 3) {
+        QString lm = helpIdPieces.value(2);
+        helpIdPieces.removeAt(2);
+        helpIdCandidates += helpIdPieces.join('.');
+
+        helpIdPieces.replace(1, lm);
+        helpIdCandidates += helpIdPieces.join('.');
+    }
+
+    helpIdPieces.removeAt(1);
+    helpIdCandidates += helpIdPieces.join('.');
+
+    const HelpItem helpItem(helpIdCandidates, qName.join('.'), HelpItem::QmlComponent);
+    const HelpItem::Links links = helpItem.links();
+
+    // Check if the module name contains a major version.
+    QRegularExpression version("^([^\\d]*)(\\d+)\\.*\\d*$");
+    QRegularExpressionMatch m = version.match(moduleName);
+    if (m.hasMatch()) {
+        QMap<QString, QUrl> filteredUrlMap;
+        QStringRef maj = m.capturedRef(2);
+        for (const HelpItem::Link &link : links) {
+            QString urlModuleName = link.second.path().split('/')[1];
+            if (urlModuleName.contains(maj))
+                filteredUrlMap.insert(link.first, link.second);
         }
-        helpIdPieces.removeAt(1);
-        helpId = helpIdPieces.join(QLatin1Char('.'));
-        if (!HelpManager::linksForIdentifier(helpId).isEmpty())
-            break;
-        return false;
-    } while (0);
-    setLastHelpItemIdentified(HelpItem(helpId, qName.join(QLatin1Char('.')), HelpItem::QmlComponent));
+        if (!filteredUrlMap.isEmpty()) {
+            // Use the URL, to disambiguate different versions
+            const HelpItem helpItem(filteredUrlMap.first(),
+                                    qName.join(QLatin1Char('.')),
+                                    HelpItem::QmlComponent);
+            setLastHelpItemIdentified(helpItem);
+            return true;
+        }
+    }
+    setLastHelpItemIdentified(helpItem);
     return true;
 }
 
-void QmlJSHoverHandler::identifyMatch(TextEditorWidget *editorWidget, int pos)
+void QmlJSHoverHandler::identifyMatch(TextEditorWidget *editorWidget, int pos, ReportPriority report)
 {
+    Utils::ExecuteOnDestruction reportPriority([this, report](){ report(priority()); });
+
     reset();
 
     if (!m_modelManager)
         return;
 
-    QmlJSEditorWidget *qmlEditor = qobject_cast<QmlJSEditorWidget *>(editorWidget);
+    auto qmlEditor = qobject_cast<QmlJSEditorWidget*>(editorWidget);
     QTC_ASSERT(qmlEditor, return);
 
     const QmlJSTools::SemanticInfo &semanticInfo = qmlEditor->qmlJsEditorDocument()->semanticInfo();
@@ -202,7 +221,7 @@ void QmlJSHoverHandler::identifyMatch(TextEditorWidget *editorWidget, int pos)
     if (rangePath.isEmpty()) {
         // Is the cursor on an import? The ast path will have an UiImport
         // member in the last or second to last position!
-        AST::UiImport *import = 0;
+        AST::UiImport *import = nullptr;
         if (astPath.size() >= 1)
             import = AST::cast<AST::UiImport *>(astPath.last());
         if (!import && astPath.size() >= 2)
@@ -277,7 +296,7 @@ bool QmlJSHoverHandler::matchColorItem(const ScopeChain &scopeChain,
     if (!initializer)
         return false;
 
-    AST::UiObjectMember *member = 0;
+    AST::UiObjectMember *member = nullptr;
     for (AST::UiObjectMemberList *list = initializer->members; list; list = list->next) {
         if (posIsInSource(pos, list->member)) {
             member = list->member;
@@ -288,8 +307,8 @@ bool QmlJSHoverHandler::matchColorItem(const ScopeChain &scopeChain,
         return false;
 
     QString color;
-    const Value *value = 0;
-    if (const AST::UiScriptBinding *binding = AST::cast<const AST::UiScriptBinding *>(member)) {
+    const Value *value = nullptr;
+    if (auto binding = AST::cast<const AST::UiScriptBinding *>(member)) {
         if (binding->qualifiedId && posIsInSource(pos, binding->statement)) {
             value = scopeChain.evaluate(binding->qualifiedId);
             if (value && value->asColorValue()) {
@@ -298,8 +317,7 @@ bool QmlJSHoverHandler::matchColorItem(const ScopeChain &scopeChain,
                                binding->statement->lastSourceLocation());
             }
         }
-    } else if (const AST::UiPublicMember *publicMember =
-               AST::cast<const AST::UiPublicMember *>(member)) {
+    } else if (auto publicMember = AST::cast<const AST::UiPublicMember *>(member)) {
         if (!publicMember->name.isEmpty() && posIsInSource(pos, publicMember->statement)) {
             value = scopeChain.lookup(publicMember->name.toString());
             if (const Reference *ref = value->asReference())
@@ -412,33 +430,33 @@ void QmlJSHoverHandler::prettyPrintTooltip(const Value *value,
 static const ObjectValue *isMember(const ScopeChain &scopeChain,
                                                 AST::Node *node, QString *name)
 {
-    const ObjectValue *owningObject = 0;
-    if (AST::IdentifierExpression *identExp = AST::cast<AST::IdentifierExpression *>(node)) {
+    const ObjectValue *owningObject = nullptr;
+    if (auto identExp = AST::cast<const AST::IdentifierExpression *>(node)) {
         if (identExp->name.isEmpty())
-            return 0;
+            return nullptr;
         *name = identExp->name.toString();
         scopeChain.lookup(*name, &owningObject);
-    } else if (AST::FieldMemberExpression *fme = AST::cast<AST::FieldMemberExpression *>(node)) {
+    } else if (auto fme = AST::cast<const AST::FieldMemberExpression *>(node)) {
         if (!fme->base || fme->name.isEmpty())
-            return 0;
+            return nullptr;
         *name = fme->name.toString();
         const Value *base = scopeChain.evaluate(fme->base);
         if (!base)
-            return 0;
+            return nullptr;
         owningObject = base->asObjectValue();
         if (owningObject)
             owningObject->lookupMember(*name, scopeChain.context(), &owningObject);
-    } else if (AST::UiQualifiedId *qid = AST::cast<AST::UiQualifiedId *>(node)) {
+    } else if (auto qid = AST::cast<const AST::UiQualifiedId *>(node)) {
         if (qid->name.isEmpty())
-            return 0;
+            return nullptr;
         *name = qid->name.toString();
         const Value *value = scopeChain.lookup(*name, &owningObject);
         for (AST::UiQualifiedId *it = qid->next; it; it = it->next) {
             if (!value)
-                return 0;
+                return nullptr;
             const ObjectValue *next = value->asObjectValue();
             if (!next || it->name.isEmpty())
-                return 0;
+                return nullptr;
             *name = it->name.toString();
             value = next->lookupMember(*name, scopeChain.context(), &owningObject);
         }
@@ -455,7 +473,7 @@ bool QmlJSHoverHandler::setQmlHelpItem(const ScopeChain &scopeChain,
     if (const ObjectValue *scope = isMember(scopeChain, node, &name)) {
         // maybe it's a type?
         if (!name.isEmpty() && name.at(0).isUpper()) {
-            if (AST::UiQualifiedId *qualifiedId = AST::cast<AST::UiQualifiedId *>(node)) {
+            if (auto qualifiedId = AST::cast<AST::UiQualifiedId *>(node)) {
                 const ObjectValue *value = scopeChain.context()->lookupType(qmlDocument.data(), qualifiedId);
                 if (setQmlTypeHelp(scopeChain, qmlDocument, value, QStringList(qualifiedId->name.toString())))
                     return true;
@@ -472,22 +490,13 @@ bool QmlJSHoverHandler::setQmlHelpItem(const ScopeChain &scopeChain,
             const QString className = cur->className();
             if (!className.isEmpty()) {
                 moduleName = getModuleName(scopeChain, qmlDocument, cur);
-                QString helpId;
-                do {
-                    helpId = QLatin1String("QML.") + moduleName + QLatin1Char('.') + className
-                            + QLatin1String("::") + name;
-                    if (!HelpManager::linksForIdentifier(helpId).isEmpty())
-                        break;
-                    helpId = QLatin1String("QML.") + className + QLatin1String("::") + name;
-                    if (!HelpManager::linksForIdentifier(helpId).isEmpty())
-                        break;
-                    helpId = className + QLatin1String("::") + name;
-                    if (!HelpManager::linksForIdentifier(helpId).isEmpty())
-                        break;
-                    helpId.clear();
-                } while (0);
-                if (!helpId.isEmpty()) {
-                    setLastHelpItemIdentified(HelpItem(helpId, name, HelpItem::QmlProperty));
+                const QStringList helpIdCandidates = {"QML." + moduleName + '.' + className
+                                                          + "::" + name,
+                                                      "QML." + className + "::" + name,
+                                                      className + "::" + name};
+                const HelpItem helpItem(helpIdCandidates, name, HelpItem::QmlProperty);
+                if (helpItem.isValid()) {
+                    setLastHelpItemIdentified(helpItem);
                     return true;
                 }
             }
@@ -497,7 +506,7 @@ bool QmlJSHoverHandler::setQmlHelpItem(const ScopeChain &scopeChain,
         }
     } else {
         // it might be a type, but the scope chain is broken (mismatched braces)
-        if (AST::UiQualifiedId *qualifiedId = AST::cast<AST::UiQualifiedId *>(node)) {
+        if (auto qualifiedId = AST::cast<AST::UiQualifiedId *>(node)) {
             const ObjectValue *value = scopeChain.context()->lookupType(qmlDocument.data(),
                                                                         qualifiedId);
             if (setQmlTypeHelp(scopeChain, qmlDocument, value,
