@@ -30,7 +30,10 @@
 //temp
 #include "qmljsquickfix.h"
 
-#include <extensionsystem/pluginmanager.h>
+#include <texteditor/codeassist/genericproposal.h>
+#include <texteditor/codeassist/iassistprocessor.h>
+
+#include <utils/algorithm.h>
 
 using namespace QmlJSTools;
 using namespace TextEditor;
@@ -40,7 +43,7 @@ namespace QmlJSEditor {
 using namespace Internal;
 
 // -----------------------
-// QuickFixAssistInterface
+// QmlJSQuickFixAssistInterface
 // -----------------------
 QmlJSQuickFixAssistInterface::QmlJSQuickFixAssistInterface(QmlJSEditorWidget *editor,
                                                            AssistReason reason)
@@ -50,8 +53,7 @@ QmlJSQuickFixAssistInterface::QmlJSQuickFixAssistInterface(QmlJSEditorWidget *ed
     , m_currentFile(QmlJSRefactoringChanges::file(editor, m_semanticInfo.document))
 {}
 
-QmlJSQuickFixAssistInterface::~QmlJSQuickFixAssistInterface()
-{}
+QmlJSQuickFixAssistInterface::~QmlJSQuickFixAssistInterface() = default;
 
 const SemanticInfo &QmlJSQuickFixAssistInterface::semanticInfo() const
 {
@@ -63,37 +65,29 @@ QmlJSRefactoringFilePtr QmlJSQuickFixAssistInterface::currentFile() const
     return m_currentFile;
 }
 
+// ---------------------------
+// QmlJSQuickFixAssistProcessor
+// ---------------------------
+class QmlJSQuickFixAssistProcessor : public IAssistProcessor
+{
+    IAssistProposal *perform(const AssistInterface *interface) override
+    {
+        return GenericProposal::createProposal(interface, findQmlJSQuickFixes(interface));
+    }
+};
+
 // ---------------------------
 // QmlJSQuickFixAssistProvider
 // ---------------------------
-QmlJSQuickFixAssistProvider::QmlJSQuickFixAssistProvider(QObject *parent)
-    : TextEditor::QuickFixAssistProvider(parent)
-{}
-
-QmlJSQuickFixAssistProvider::~QmlJSQuickFixAssistProvider()
-{}
 
 IAssistProvider::RunType QmlJSQuickFixAssistProvider::runType() const
 {
     return Synchronous;
 }
 
-bool QmlJSQuickFixAssistProvider::supportsEditor(Core::Id editorId) const
-{
-    return editorId == Constants::C_QMLJSEDITOR_ID;
-}
-
 IAssistProcessor *QmlJSQuickFixAssistProvider::createProcessor() const
 {
-    return new QuickFixAssistProcessor(this);
-}
-
-QList<QuickFixFactory *> QmlJSQuickFixAssistProvider::quickFixFactories() const
-{
-    QList<QuickFixFactory *> results;
-    foreach (QmlJSQuickFixFactory *f, ExtensionSystem::PluginManager::getObjects<QmlJSQuickFixFactory>())
-        results.append(f);
-    return results;
+    return new QmlJSQuickFixAssistProcessor;
 }
 
 } // namespace QmlJSEditor
