@@ -25,16 +25,15 @@
 
 #pragma once
 
-#include "subversionsettings.h"
 #include <vcsbase/vcsbaseclient.h>
-#include <vcsbase/vcscommand.h>
 
 #include <utils/fileutils.h>
 
+namespace VcsBase { class VcsCommand; }
+
 namespace Subversion {
 namespace Internal {
 
-class SubversionSettings;
 class SubversionDiffEditorController;
 
 class SubversionClient : public VcsBase::VcsBaseClient
@@ -44,10 +43,10 @@ class SubversionClient : public VcsBase::VcsBaseClient
 public:
     SubversionClient();
 
-    VcsBase::VcsCommand *createCommitCmd(const QString &repositoryRoot,
-                                         const QStringList &files,
-                                         const QString &commitMessageFile,
-                                         const QStringList &extraOptions = QStringList()) const;
+    bool doCommit(const QString &repositoryRoot,
+                  const QStringList &files,
+                  const QString &commitMessageFile,
+                  const QStringList &extraOptions = QStringList()) const;
     void commit(const QString &repositoryRoot,
                 const QStringList &files,
                 const QString &commitMessageFile,
