@@ -89,14 +89,19 @@ QVariant ModelAdapter::data(const QModelIndex &index, int role) const
     if (!index.isValid() || index.row() >= m_completionModel->size())
         return QVariant();
 
-    if (role == Qt::DisplayRole)
-        return m_completionModel->text(index.row());
-    else if (role == Qt::DecorationRole)
+    if (role == Qt::DisplayRole) {
+        const QString text = m_completionModel->text(index.row());
+        const int lineBreakPos = text.indexOf('\n');
+        if (lineBreakPos < 0)
+            return text;
+        return QString(text.leftRef(lineBreakPos) + QLatin1String(" (...)"));
+    } else if (role == Qt::DecorationRole) {
         return m_completionModel->icon(index.row());
-    else if (role == Qt::WhatsThisRole)
+    } else if (role == Qt::WhatsThisRole) {
         return m_completionModel->detail(index.row());
-    else if (role == Qt::UserRole)
+    } else if (role == Qt::UserRole) {
         return m_completionModel->proposalItem(index.row())->requiresFixIts();
+    }
 
     return QVariant();
 }
@@ -554,6 +559,7 @@ bool GenericProposalWidget::eventFilter(QObject *o, QEvent *e)
         switch (ke->key()) {
         case Qt::Key_N:
         case Qt::Key_P:
+        case Qt::Key_BracketLeft:
             if (ke->modifiers() == Qt::KeyboardModifiers(HostOsInfo::controlModifier())) {
                 e->accept();
                 return true;
@@ -568,6 +574,16 @@ bool GenericProposalWidget::eventFilter(QObject *o, QEvent *e)
             e->accept();
             return true;
 
+        case Qt::Key_BracketLeft:
+            // vim-style behavior
+            if (ke->modifiers() == Qt::KeyboardModifiers(HostOsInfo::controlModifier())) {
+                abort();
+                emit explicitlyAborted();
+                e->accept();
+                return true;
+            }
+            break;
+
         case Qt::Key_N:
         case Qt::Key_P:
             // select next/previous completion
