@@ -50,6 +50,8 @@
 #include <QVariant>
 #include <QProcessEnvironment>
 
+using namespace Utils;
+
 /*!
     \class VcsBase::VcsBaseClient
 
@@ -65,8 +67,8 @@ static Core::IEditor *locateEditor(const char *property, const QString &entry)
 {
     foreach (Core::IDocument *document, Core::DocumentModel::openedDocuments())
         if (document->property(property).toString() == entry)
-            return Core::DocumentModel::editorsForDocument(document).first();
-    return 0;
+            return Core::DocumentModel::editorsForDocument(document).constFirst();
+    return nullptr;
 }
 
 namespace VcsBase {
@@ -108,7 +110,7 @@ VcsBaseClientSettings &VcsBaseClientImpl::settings() const
     return *d->m_clientSettings;
 }
 
-Utils::FileName VcsBaseClientImpl::vcsBinary() const
+FileName VcsBaseClientImpl::vcsBinary() const
 {
     return settings().binaryPath();
 }
@@ -134,7 +136,7 @@ VcsCommand *VcsBaseClientImpl::createCommand(const QString &workingDirectory,
 
 void VcsBaseClientImpl::enqueueJob(VcsCommand *cmd, const QStringList &args,
                                    const QString &workingDirectory,
-                                   const Utils::ExitCodeInterpreter &interpreter) const
+                                   const ExitCodeInterpreter &interpreter) const
 {
     cmd->addJob(vcsBinary(), args, vcsTimeoutS(), workingDirectory, interpreter);
     cmd->execute();
@@ -149,7 +151,7 @@ QProcessEnvironment VcsBaseClientImpl::processEnvironment() const
 
 QString VcsBaseClientImpl::commandOutputFromLocal8Bit(const QByteArray &a)
 {
-    return Utils::SynchronousProcess::normalizeNewlines(QString::fromLocal8Bit(a));
+    return SynchronousProcess::normalizeNewlines(QString::fromLocal8Bit(a));
 }
 
 QStringList VcsBaseClientImpl::commandOutputLinesFromLocal8Bit(const QByteArray &a)
@@ -175,8 +177,8 @@ QString VcsBaseClientImpl::stripLastNewline(const QString &in)
     return in;
 }
 
-Utils::SynchronousProcessResponse
-VcsBaseClientImpl::vcsFullySynchronousExec(const QString &workingDir, const Utils::FileName &binary,
+SynchronousProcessResponse
+VcsBaseClientImpl::vcsFullySynchronousExec(const QString &workingDir, const FileName &binary,
                                            const QStringList &args, unsigned flags,
                                            int timeoutS, QTextCodec *codec) const
 {
@@ -205,7 +207,7 @@ void VcsBaseClientImpl::annotateRevisionRequested(const QString &workingDirector
     annotate(workingDirectory, file, changeCopy, line);
 }
 
-Utils::SynchronousProcessResponse
+SynchronousProcessResponse
 VcsBaseClientImpl::vcsFullySynchronousExec(const QString &workingDir, const QStringList &args,
                                            unsigned flags, int timeoutS, QTextCodec *codec) const
 {
@@ -226,10 +228,10 @@ VcsCommand *VcsBaseClientImpl::vcsExec(const QString &workingDirectory, const QS
     return command;
 }
 
-Utils::SynchronousProcessResponse VcsBaseClientImpl::vcsSynchronousExec(const QString &workingDir,
-                                                                        const QStringList &args,
-                                                                        unsigned flags,
-                                                                        QTextCodec *outputCodec) const
+SynchronousProcessResponse VcsBaseClientImpl::vcsSynchronousExec(const QString &workingDir,
+                                                                 const QStringList &args,
+                                                                 unsigned flags,
+                                                                 QTextCodec *outputCodec) const
 {
     return VcsBasePlugin::runVcs(workingDir, vcsBinary(), args, vcsTimeoutS(), flags,
                                  outputCodec, processEnvironment());
@@ -245,20 +247,20 @@ VcsBaseEditorWidget *VcsBaseClientImpl::createVcsEditor(Core::Id kind, QString t
                                                         const char *registerDynamicProperty,
                                                         const QString &dynamicPropertyValue) const
 {
-    VcsBaseEditorWidget *baseEditor = 0;
+    VcsBaseEditorWidget *baseEditor = nullptr;
     Core::IEditor *outputEditor = locateEditor(registerDynamicProperty, dynamicPropertyValue);
     const QString progressMsg = tr("Working...");
     if (outputEditor) {
         // Exists already
         outputEditor->document()->setContents(progressMsg.toUtf8());
         baseEditor = VcsBaseEditor::getVcsBaseEditor(outputEditor);
-        QTC_ASSERT(baseEditor, return 0);
+        QTC_ASSERT(baseEditor, return nullptr);
         Core::EditorManager::activateEditor(outputEditor);
     } else {
         outputEditor = Core::EditorManager::openEditorWithContents(kind, &title, progressMsg.toUtf8());
         outputEditor->document()->setProperty(registerDynamicProperty, dynamicPropertyValue);
         baseEditor = VcsBaseEditor::getVcsBaseEditor(outputEditor);
-        QTC_ASSERT(baseEditor, return 0);
+        QTC_ASSERT(baseEditor, return nullptr);
         connect(baseEditor, &VcsBaseEditorWidget::annotateRevisionRequested,
                 this, &VcsBaseClientImpl::annotateRevisionRequested);
         baseEditor->setSource(source);
@@ -316,8 +318,8 @@ bool VcsBaseClient::synchronousCreateRepository(const QString &workingDirectory,
 {
     QStringList args(vcsCommandString(CreateRepositoryCommand));
     args << extraOptions;
-    Utils::SynchronousProcessResponse result = vcsFullySynchronousExec(workingDirectory, args);
-    if (result.result != Utils::SynchronousProcessResponse::Finished)
+    SynchronousProcessResponse result = vcsFullySynchronousExec(workingDirectory, args);
+    if (result.result != SynchronousProcessResponse::Finished)
         return false;
     VcsOutputWindow::append(result.stdOut());
 
@@ -335,9 +337,9 @@ bool VcsBaseClient::synchronousClone(const QString &workingDir,
     args << vcsCommandString(CloneCommand)
          << extraOptions << srcLocation << dstLocation;
 
-    Utils::SynchronousProcessResponse result = vcsFullySynchronousExec(workingDir, args);
+    SynchronousProcessResponse result = vcsFullySynchronousExec(workingDir, args);
     resetCachedVcsInfo(workingDir);
-    return result.result == Utils::SynchronousProcessResponse::Finished;
+    return result.result == SynchronousProcessResponse::Finished;
 }
 
 bool VcsBaseClient::synchronousAdd(const QString &workingDir, const QString &filename,
@@ -345,7 +347,7 @@ bool VcsBaseClient::synchronousAdd(const QString &workingDir, const QString &fil
 {
     QStringList args;
     args << vcsCommandString(AddCommand) << extraOptions << filename;
-    return vcsFullySynchronousExec(workingDir, args).result == Utils::SynchronousProcessResponse::Finished;
+    return vcsFullySynchronousExec(workingDir, args).result == SynchronousProcessResponse::Finished;
 }
 
 bool VcsBaseClient::synchronousRemove(const QString &workingDir, const QString &filename,
@@ -353,7 +355,7 @@ bool VcsBaseClient::synchronousRemove(const QString &workingDir, const QString &
 {
     QStringList args;
     args << vcsCommandString(RemoveCommand) << extraOptions << filename;
-    return vcsFullySynchronousExec(workingDir, args).result == Utils::SynchronousProcessResponse::Finished;
+    return vcsFullySynchronousExec(workingDir, args).result == SynchronousProcessResponse::Finished;
 }
 
 bool VcsBaseClient::synchronousMove(const QString &workingDir,
@@ -362,7 +364,7 @@ bool VcsBaseClient::synchronousMove(const QString &workingDir,
 {
     QStringList args;
     args << vcsCommandString(MoveCommand) << extraOptions << from << to;
-    return vcsFullySynchronousExec(workingDir, args).result == Utils::SynchronousProcessResponse::Finished;
+    return vcsFullySynchronousExec(workingDir, args).result == SynchronousProcessResponse::Finished;
 }
 
 bool VcsBaseClient::synchronousPull(const QString &workingDir,
@@ -376,8 +378,8 @@ bool VcsBaseClient::synchronousPull(const QString &workingDir,
             VcsCommand::SshPasswordPrompt
             | VcsCommand::ShowStdOut
             | VcsCommand::ShowSuccessMessage;
-    const Utils::SynchronousProcessResponse resp = vcsSynchronousExec(workingDir, args, flags);
-    const bool ok = resp.result == Utils::SynchronousProcessResponse::Finished;
+    const SynchronousProcessResponse resp = vcsSynchronousExec(workingDir, args, flags);
+    const bool ok = resp.result == SynchronousProcessResponse::Finished;
     if (ok)
         emit changed(QVariant(workingDir));
     return ok;
@@ -394,8 +396,8 @@ bool VcsBaseClient::synchronousPush(const QString &workingDir,
             VcsCommand::SshPasswordPrompt
             | VcsCommand::ShowStdOut
             | VcsCommand::ShowSuccessMessage;
-    const Utils::SynchronousProcessResponse resp = vcsSynchronousExec(workingDir, args, flags);
-    return resp.result == Utils::SynchronousProcessResponse::Finished;
+    const SynchronousProcessResponse resp = vcsSynchronousExec(workingDir, args, flags);
+    return resp.result == SynchronousProcessResponse::Finished;
 }
 
 VcsBaseEditorWidget *VcsBaseClient::annotate(
@@ -453,7 +455,8 @@ void VcsBaseClient::diff(const QString &workingDir, const QStringList &files,
     else
         args << extraOptions;
     args << files;
-    QTextCodec *codec = source.isEmpty() ? static_cast<QTextCodec *>(0) : VcsBaseEditor::getCodec(source);
+    QTextCodec *codec = source.isEmpty() ? static_cast<QTextCodec *>(nullptr)
+                                         : VcsBaseEditor::getCodec(source);
     VcsCommand *command = createCommand(workingDir, editor);
     command->setCodec(codec);
     enqueueJob(command, args, workingDir, exitCodeInterpreter(DiffCommand));
@@ -526,7 +529,7 @@ void VcsBaseClient::status(const QString &workingDir, const QString &file,
     QStringList args(vcsCommandString(StatusCommand));
     args << extraOptions << file;
     VcsOutputWindow::setRepository(workingDir);
-    VcsCommand *cmd = createCommand(workingDir, 0, VcsWindowOutputBind);
+    VcsCommand *cmd = createCommand(workingDir, nullptr, VcsWindowOutputBind);
     connect(cmd, &VcsCommand::finished,
             VcsOutputWindow::instance(), &VcsOutputWindow::clearRepository,
             Qt::QueuedConnection);
@@ -564,7 +567,7 @@ QString VcsBaseClient::vcsCommandString(VcsCommandTag cmd) const
     return QString();
 }
 
-Utils::ExitCodeInterpreter VcsBaseClient::exitCodeInterpreter(VcsCommandTag cmd) const
+ExitCodeInterpreter VcsBaseClient::exitCodeInterpreter(VcsCommandTag cmd) const
 {
     Q_UNUSED(cmd)
     return Utils::defaultExitCodeInterpreter;
@@ -630,7 +633,7 @@ void VcsBaseClient::commit(const QString &repositoryRoot,
     //   for example)
     QStringList args(vcsCommandString(CommitCommand));
     args << extraOptions << files;
-    VcsCommand *cmd = createCommand(repositoryRoot, 0, VcsWindowOutputBind);
+    VcsCommand *cmd = createCommand(repositoryRoot, nullptr, VcsWindowOutputBind);
     if (!commitMessageFile.isEmpty())
         connect(cmd, &VcsCommand::finished, [commitMessageFile]() { QFile(commitMessageFile).remove(); });
     enqueueJob(cmd, args);
@@ -640,7 +643,7 @@ QString VcsBaseClient::vcsEditorTitle(const QString &vcsCmd, const QString &sour
 {
     return vcsBinary().toFileInfo().baseName() +
             QLatin1Char(' ') + vcsCmd + QLatin1Char(' ') +
-            Utils::FileName::fromString(sourceId).fileName();
+            FileName::fromString(sourceId).fileName();
 }
 
 void VcsBaseClient::statusParser(const QString &text)
